#line 1 "Engine/csound_prslex.c"

#line 3 "Engine/csound_prslex.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

/* %not-for-header */
/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */
/* %ok-for-header */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 6
#define YY_FLEX_SUBMINOR_VERSION 4
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* %if-c++-only */
/* %endif */

/* %if-c-only */
#ifdef yy_create_buffer
#define csound_prs_create_buffer_ALREADY_DEFINED
#else
#define yy_create_buffer csound_prs_create_buffer
#endif

#ifdef yy_delete_buffer
#define csound_prs_delete_buffer_ALREADY_DEFINED
#else
#define yy_delete_buffer csound_prs_delete_buffer
#endif

#ifdef yy_scan_buffer
#define csound_prs_scan_buffer_ALREADY_DEFINED
#else
#define yy_scan_buffer csound_prs_scan_buffer
#endif

#ifdef yy_scan_string
#define csound_prs_scan_string_ALREADY_DEFINED
#else
#define yy_scan_string csound_prs_scan_string
#endif

#ifdef yy_scan_bytes
#define csound_prs_scan_bytes_ALREADY_DEFINED
#else
#define yy_scan_bytes csound_prs_scan_bytes
#endif

#ifdef yy_init_buffer
#define csound_prs_init_buffer_ALREADY_DEFINED
#else
#define yy_init_buffer csound_prs_init_buffer
#endif

#ifdef yy_flush_buffer
#define csound_prs_flush_buffer_ALREADY_DEFINED
#else
#define yy_flush_buffer csound_prs_flush_buffer
#endif

#ifdef yy_load_buffer_state
#define csound_prs_load_buffer_state_ALREADY_DEFINED
#else
#define yy_load_buffer_state csound_prs_load_buffer_state
#endif

#ifdef yy_switch_to_buffer
#define csound_prs_switch_to_buffer_ALREADY_DEFINED
#else
#define yy_switch_to_buffer csound_prs_switch_to_buffer
#endif

#ifdef yypush_buffer_state
#define csound_prspush_buffer_state_ALREADY_DEFINED
#else
#define yypush_buffer_state csound_prspush_buffer_state
#endif

#ifdef yypop_buffer_state
#define csound_prspop_buffer_state_ALREADY_DEFINED
#else
#define yypop_buffer_state csound_prspop_buffer_state
#endif

#ifdef yyensure_buffer_stack
#define csound_prsensure_buffer_stack_ALREADY_DEFINED
#else
#define yyensure_buffer_stack csound_prsensure_buffer_stack
#endif

#ifdef yylex
#define csound_prslex_ALREADY_DEFINED
#else
#define yylex csound_prslex
#endif

#ifdef yyrestart
#define csound_prsrestart_ALREADY_DEFINED
#else
#define yyrestart csound_prsrestart
#endif

#ifdef yylex_init
#define csound_prslex_init_ALREADY_DEFINED
#else
#define yylex_init csound_prslex_init
#endif

#ifdef yylex_init_extra
#define csound_prslex_init_extra_ALREADY_DEFINED
#else
#define yylex_init_extra csound_prslex_init_extra
#endif

#ifdef yylex_destroy
#define csound_prslex_destroy_ALREADY_DEFINED
#else
#define yylex_destroy csound_prslex_destroy
#endif

#ifdef yyget_debug
#define csound_prsget_debug_ALREADY_DEFINED
#else
#define yyget_debug csound_prsget_debug
#endif

#ifdef yyset_debug
#define csound_prsset_debug_ALREADY_DEFINED
#else
#define yyset_debug csound_prsset_debug
#endif

#ifdef yyget_extra
#define csound_prsget_extra_ALREADY_DEFINED
#else
#define yyget_extra csound_prsget_extra
#endif

#ifdef yyset_extra
#define csound_prsset_extra_ALREADY_DEFINED
#else
#define yyset_extra csound_prsset_extra
#endif

#ifdef yyget_in
#define csound_prsget_in_ALREADY_DEFINED
#else
#define yyget_in csound_prsget_in
#endif

#ifdef yyset_in
#define csound_prsset_in_ALREADY_DEFINED
#else
#define yyset_in csound_prsset_in
#endif

#ifdef yyget_out
#define csound_prsget_out_ALREADY_DEFINED
#else
#define yyget_out csound_prsget_out
#endif

#ifdef yyset_out
#define csound_prsset_out_ALREADY_DEFINED
#else
#define yyset_out csound_prsset_out
#endif

#ifdef yyget_leng
#define csound_prsget_leng_ALREADY_DEFINED
#else
#define yyget_leng csound_prsget_leng
#endif

#ifdef yyget_text
#define csound_prsget_text_ALREADY_DEFINED
#else
#define yyget_text csound_prsget_text
#endif

#ifdef yyget_lineno
#define csound_prsget_lineno_ALREADY_DEFINED
#else
#define yyget_lineno csound_prsget_lineno
#endif

#ifdef yyset_lineno
#define csound_prsset_lineno_ALREADY_DEFINED
#else
#define yyset_lineno csound_prsset_lineno
#endif

#ifdef yyget_column
#define csound_prsget_column_ALREADY_DEFINED
#else
#define yyget_column csound_prsget_column
#endif

#ifdef yyset_column
#define csound_prsset_column_ALREADY_DEFINED
#else
#define yyset_column csound_prsset_column
#endif

#ifdef yywrap
#define csound_prswrap_ALREADY_DEFINED
#else
#define yywrap csound_prswrap
#endif

/* %endif */

#ifdef yyalloc
#define csound_prsalloc_ALREADY_DEFINED
#else
#define yyalloc csound_prsalloc
#endif

#ifdef yyrealloc
#define csound_prsrealloc_ALREADY_DEFINED
#else
#define yyrealloc csound_prsrealloc
#endif

#ifdef yyfree
#define csound_prsfree_ALREADY_DEFINED
#else
#define yyfree csound_prsfree
#endif

/* %if-c-only */

/* %endif */

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* %if-c-only */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
/* %endif */

/* %if-tables-serialization */
/* %endif */
/* end standard C headers. */

/* %if-c-or-c++ */
/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#ifndef SIZE_MAX
#define SIZE_MAX               (~(size_t)0)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

/* %endif */

/* begin standard C++ headers. */
/* %if-c++-only */
/* %endif */

/* TODO: this is always defined, so inline it */
#define yyconst const

#if defined(__GNUC__) && __GNUC__ >= 3
#define yynoreturn __attribute__((__noreturn__))
#else
#define yynoreturn
#endif

/* %not-for-header */
/* Returned upon end-of-file. */
#define YY_NULL 0
/* %ok-for-header */

/* %not-for-header */
/* Promotes a possibly negative, possibly signed char to an
 *   integer in range [0..255] for use as an array index.
 */
#define YY_SC_TO_UI(c) ((YY_CHAR) (c))
/* %ok-for-header */

/* %if-reentrant */

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r

/* %endif */

/* %if-not-reentrant */
/* %endif */

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *
/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START
/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)
/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin , yyscanner )
#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

/* %if-not-reentrant */
/* %endif */

/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2
    
    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = yyg->yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )
#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
/* %if-c-only */
	FILE *yy_input_file;
/* %endif */

/* %if-c++-only */
/* %endif */

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */
/* %if-not-reentrant */
/* %endif */
/* %ok-for-header */

/* %endif */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                          ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                          : NULL)
/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]

/* %if-c-only Standard (non-C++) definition */

/* %if-not-reentrant */
/* %not-for-header */
/* %ok-for-header */

/* %endif */

void yyrestart ( FILE *input_file , yyscan_t yyscanner );
void yy_switch_to_buffer ( YY_BUFFER_STATE new_buffer , yyscan_t yyscanner );
YY_BUFFER_STATE yy_create_buffer ( FILE *file, int size , yyscan_t yyscanner );
void yy_delete_buffer ( YY_BUFFER_STATE b , yyscan_t yyscanner );
void yy_flush_buffer ( YY_BUFFER_STATE b , yyscan_t yyscanner );
void yypush_buffer_state ( YY_BUFFER_STATE new_buffer , yyscan_t yyscanner );
void yypop_buffer_state ( yyscan_t yyscanner );

static void yyensure_buffer_stack ( yyscan_t yyscanner );
static void yy_load_buffer_state ( yyscan_t yyscanner );
static void yy_init_buffer ( YY_BUFFER_STATE b, FILE *file , yyscan_t yyscanner );
#define YY_FLUSH_BUFFER yy_flush_buffer( YY_CURRENT_BUFFER , yyscanner)

YY_BUFFER_STATE yy_scan_buffer ( char *base, yy_size_t size , yyscan_t yyscanner );
YY_BUFFER_STATE yy_scan_string ( const char *yy_str , yyscan_t yyscanner );
YY_BUFFER_STATE yy_scan_bytes ( const char *bytes, int len , yyscan_t yyscanner );

/* %endif */

void *yyalloc ( yy_size_t , yyscan_t yyscanner );
void *yyrealloc ( void *, yy_size_t , yyscan_t yyscanner );
void yyfree ( void * , yyscan_t yyscanner );

#define yy_new_buffer yy_create_buffer
#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        yyensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}
#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        yyensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}
#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* %% [1.0] yytext/yyin/yyout/yy_state_type/yylineno etc. def's & init go here */
/* Begin user sect3 */

#define csound_prswrap(yyscanner) (/*CONSTCOND*/1)
#define YY_SKIP_YYWRAP

#define FLEX_DEBUG
typedef flex_uint8_t YY_CHAR;

typedef int yy_state_type;

#define yytext_ptr yytext_r

/* %% [1.5] DFA */

/* %if-c-only Standard (non-C++) definition */

static yy_state_type yy_get_previous_state ( yyscan_t yyscanner );
static yy_state_type yy_try_NUL_trans ( yy_state_type current_state  , yyscan_t yyscanner);
static int yy_get_next_buffer ( yyscan_t yyscanner );
static void yynoreturn yy_fatal_error ( const char* msg , yyscan_t yyscanner );

/* %endif */

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yyg->yytext_ptr = yy_bp; \
/* %% [2.0] code to fiddle yytext and yyleng for yymore() goes here \ */\
	yyleng = (int) (yy_cp - yy_bp); \
	yyg->yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
/* %% [3.0] code to copy yytext_ptr to yytext[] goes here, if %array \ */\
	yyg->yy_c_buf_p = yy_cp;
/* %% [4.0] data tables for the DFA and the user's section 1 definitions go here */
#define YY_NUM_RULES 39
#define YY_END_OF_BUFFER 40
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static const flex_int16_t yy_accept[132] =
    {   0,
        0,    0,   13,   13,   18,   18,   23,   23,   26,   26,
       31,   31,   40,   38,    2,    2,    8,   38,   38,   38,
        4,   38,    5,   38,   38,   33,   34,   38,   37,   36,
       14,   13,   39,   21,   18,   20,   23,   24,   26,   27,
       31,   32,    2,    0,    0,    0,    0,    0,    0,    9,
        0,    6,    3,    7,    7,    1,    1,    7,   30,    0,
        0,   37,   36,   13,   18,   19,   20,   23,   24,   26,
       27,   31,   32,    0,    0,    0,    0,    0,    0,    0,
        0,   10,    9,    9,    0,    1,    0,    0,    1,   15,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    1,

        0,    0,    0,   29,   29,    0,    0,   16,    0,    0,
       35,    0,    0,    0,   28,    0,    0,   29,    0,   25,
        0,   22,   17,    0,   29,    0,   12,    0,    0,   11,
        0
    } ;

static const YY_CHAR yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    4,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    5,    6,    7,    1,    1,    1,    8,
        1,    9,    1,    1,    1,   10,   11,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,    1,   13,    1,
        1,    1,    1,    1,   14,   14,   14,   14,   14,   14,
       14,   14,   14,   14,   14,   14,   14,   14,   14,   14,
       14,   14,   14,   14,   14,   14,   14,   14,   14,   14,
        1,   15,    1,    1,   14,   16,   14,   14,   17,   18,

       19,   20,   14,   14,   21,   14,   14,   22,   23,   24,
       25,   26,   14,   27,   28,   29,   30,   14,   14,   31,
       14,   14,   32,    1,   33,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static const YY_CHAR yy_meta[34] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    2,    1,    3,
        1,    4,    1,    5,    1,    6,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    1,    1
    } ;

static const flex_int16_t yy_base[144] =
    {   0,
        0,   33,    6,   16,   66,    0,   99,    0,  132,    0,
      165,    0,  367,  368,  368,  363,  368,  197,  349,    1,
      368,   18,  368,  362,    0,  368,  368,  198,  368,  368,
      368,  361,  368,  368,  360,  353,  358,    0,  357,    0,
      356,    0,  368,   12,  338,  179,    5,  331,  331,  194,
      338,  368,  368,  368,  209,  368,  350,   24,  350,  339,
      204,  368,  368,  348,  347,  368,  340,  345,    0,  344,
        0,  343,    0,  324,  315,  324,  320,  190,  323,  314,
      320,  368,  329,  197,  226,  333,  217,  230,  232,  323,
      313,  314,  238,  303,  312,  312,  307,  302,  308,  368,

      302,  235,  241,  368,  291,  243,  251,  368,  250,  239,
      368,  248,  248,  247,  368,  263,  249,  252,  259,  368,
      247,  368,  368,  261,  368,  239,  229,  195,  182,  368,
      368,  272,  278,  280,  286,  291,  293,  295,  297,  301,
      307,  313,  319
    } ;

static const flex_int16_t yy_def[144] =
    {   0,
      132,  131,  133,  133,  131,    5,  131,    7,  131,    9,
      131,   11,  131,  131,  131,  131,  131,  131,  134,  131,
      131,  135,  131,  131,  131,  131,  131,  131,  131,  131,
      131,  131,  131,  131,  131,  136,  131,  137,  131,  138,
      131,  139,  131,  131,  131,  131,  131,  131,  131,  140,
      134,  131,  131,  131,  131,  131,  131,  141,  131,  131,
      131,  131,  131,  131,  131,  131,  136,  131,  137,  131,
      138,  131,  139,  131,  131,  131,  131,  131,  131,  131,
      131,  131,  131,  140,  131,  131,  141,  141,  141,  131,
      131,  131,  131,  131,  131,  131,  131,  131,  131,  131,

      131,  131,  131,  131,  131,  142,  131,  131,  131,  131,
      131,  131,  131,  131,  131,  143,  142,  142,  131,  131,
      131,  131,  131,  143,  131,  131,  131,  131,  131,  131,
        0,  131,  131,  131,  131,  131,  131,  131,  131,  131,
      131,  131,  131
    } ;

static const flex_int16_t yy_nxt[402] =
    {   0,
      131,   59,   15,   16,   17,   18,   19,   32,   33,   52,
       20,   53,   21,   44,   22,  131,   23,   32,   33,   55,
       56,   57,   24,   25,   78,   60,   56,   89,   79,   45,
       58,   26,   27,   14,   28,   15,   16,   17,   18,   19,
       14,   14,   14,   20,   14,   21,   14,   22,   14,   23,
       14,   29,   14,   14,   14,   24,   25,   14,   14,   30,
       29,   14,   14,   14,   26,   27,   34,   35,   33,   34,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   36,
       34,   34,   36,   36,   36,   36,   36,   36,   36,   36,
       36,   36,   36,   36,   36,   36,   36,   34,   34,   33,

       37,   33,   33,   33,   33,   33,   33,   33,   33,   33,
       33,   33,   38,   33,   33,   38,   38,   38,   38,   38,
       38,   38,   38,   38,   38,   38,   38,   38,   38,   38,
       33,   33,   33,   39,   33,   33,   33,   33,   33,   33,
       33,   33,   33,   33,   33,   40,   33,   33,   40,   40,
       40,   40,   40,   40,   40,   40,   40,   40,   40,   40,
       40,   40,   40,   33,   33,   33,   41,   33,   33,   33,
       33,   33,   33,   33,   33,   33,   33,   33,   42,   33,
       33,   42,   42,   42,   42,   42,   42,   42,   42,   42,
       42,   42,   42,   42,   42,   42,   33,   33,   44,   61,

       75,   82,   76,   83,   82,   61,   83,   95,  130,   77,
       85,   56,   86,   96,   45,   46,   62,   47,   48,   56,
       89,   87,   62,  129,   63,   62,   49,   85,   56,   86,
       63,   62,   56,   89,  100,   89,  114,  115,   87,  103,
      104,  105,  103,  104,  105,  104,  118,  116,  114,  115,
      106,  104,  118,  106,  125,  118,  128,  127,  107,  116,
      103,  104,  105,  115,  126,  115,  123,  122,  121,  120,
      119,  106,   14,   14,   14,   14,   14,   14,   31,   31,
       31,   31,   31,   31,   50,   50,   54,   54,   54,   54,
       54,   54,   67,  104,   67,   67,   69,   69,   71,   71,

       73,   73,   84,   84,   84,   84,   84,   88,   88,   88,
       88,   88,   88,  117,  117,  117,  117,  117,  117,  124,
      124,  124,  124,  124,  124,  113,  112,  111,  110,   95,
      109,  108,  102,  101,   90,   56,   82,   99,   98,   97,
       94,   93,   92,   91,   72,   70,   68,   66,   65,   64,
       90,   59,   56,  131,   81,   80,   74,   72,   70,   68,
       66,   65,   64,   59,   51,   43,  131,   13,  131,  131,
      131,  131,  131,  131,  131,  131,  131,  131,  131,  131,
      131,  131,  131,  131,  131,  131,  131,  131,  131,  131,
      131,  131,  131,  131,  131,  131,  131,  131,  131,  131,

      131
    } ;

static const flex_int16_t yy_chk[402] =
    {   0,
        0,   25,    1,    1,    1,    1,    1,    3,    3,   20,
        1,   20,    1,   44,    1,    0,    1,    4,    4,   22,
       22,   22,    1,    1,   47,   25,   58,   58,   47,   44,
       22,    1,    1,    2,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    7,

        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    9,    9,    9,    9,    9,    9,    9,    9,
        9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
        9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
        9,    9,    9,    9,    9,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   18,   28,

       46,   50,   46,   50,   84,   61,   84,   78,  129,   46,
       55,   55,   55,   78,   18,   18,   28,   18,   18,   87,
       87,   55,   61,  128,   28,   28,   18,   85,   85,   85,
       61,   61,   88,   88,   89,   89,  102,  102,   85,   93,
       93,   93,  103,  103,  103,  106,  106,  102,  114,  114,
       93,  117,  117,  103,  118,  118,  127,  126,   93,  114,
      119,  119,  119,  124,  121,  116,  113,  112,  110,  109,
      107,  119,  132,  132,  132,  132,  132,  132,  133,  133,
      133,  133,  133,  133,  134,  134,  135,  135,  135,  135,
      135,  135,  136,  105,  136,  136,  137,  137,  138,  138,

      139,  139,  140,  140,  140,  140,  140,  141,  141,  141,
      141,  141,  141,  142,  142,  142,  142,  142,  142,  143,
      143,  143,  143,  143,  143,  101,   99,   98,   97,   96,
       95,   94,   92,   91,   90,   86,   83,   81,   80,   79,
       77,   76,   75,   74,   72,   70,   68,   67,   65,   64,
       60,   59,   57,   51,   49,   48,   45,   41,   39,   37,
       36,   35,   32,   24,   19,   16,   13,  131,  131,  131,
      131,  131,  131,  131,  131,  131,  131,  131,  131,  131,
      131,  131,  131,  131,  131,  131,  131,  131,  131,  131,
      131,  131,  131,  131,  131,  131,  131,  131,  131,  131,

      131
    } ;

static const flex_int16_t yy_rule_linenum[39] =
    {   0,
      119,  132,  138,  148,  158,  168,  174,  175,  184,  196,
      214,  220,  226,  227,  231,  234,  244,  250,  251,  261,
      268,  274,  280,  281,  287,  300,  301,  305,  326,  344,
      352,  353,  359,  426,  462,  489,  562,  647
    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#line 1 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
#line 2 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"

 /*
    csound_prs.lex:

    Copyright (C) 2011, 2016
    John ffitch

    This file is part of Csound.

    The Csound Library is free software; you can redistribute it
    and/or modify it under the terms of the Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    Csound is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Csound; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
    02110-1301 USA
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "csoundCore.h"
#include "corfile.h"
#define YY_DECL int yylex (CSOUND *csound, yyscan_t yyscanner)
static void comment(yyscan_t);
static void do_comment(yyscan_t);
static void do_include(CSOUND *, int, yyscan_t);
static void do_new_include(CSOUND *, yyscan_t);
extern int isDir(char *);
static void do_macro_arg(CSOUND *, char *, yyscan_t);
static void do_macro(CSOUND *, char *, yyscan_t);
static void do_umacro(CSOUND *, char *, yyscan_t);
static void do_ifdef(CSOUND *, char *, yyscan_t);
static void do_ifdef_skip_code(CSOUND *, yyscan_t);
//static void print_csound_prsdata(CSOUND *,char *,yyscan_t);
static void csound_prs_line(CORFIL*, yyscan_t);
static void delete_macros(CSOUND*, yyscan_t);
static int extract_int(CSOUND*, yyscan_t, int*);
static int on_EOF(CSOUND*, yyscan_t);
#define MACDEBUG 1

#define S_INC (10)
static inline int isNameChar(int cc, int pos)
{
    unsigned char c = ((unsigned char) cc);
    return (isalpha(c) || (pos && (c == '_' || isdigit(c))));
}

#include "score_param.h"

static void expand_macro(CSOUND*, MACRO*, yyscan_t);
static void expand_macroa(CSOUND*, MACRO*, yyscan_t);
//static void trace_alt_stack(CSOUND*, PRS_PARM*, int);

#define YY_EXTRA_TYPE  PRS_PARM *
#define PARM    yyget_extra(yyscanner)

#define YY_USER_INIT {csound_prs_scan_string(csound->scorestr->body, yyscanner); \
    csound_prsset_lineno(csound->scoLineOffset, yyscanner);             \
    /* yyg->yy_flex_debug_r=1;*/                                        \
    PARM->macro_stack_size = 0;                                         \
    PARM->alt_stack = NULL; PARM->macro_stack_ptr = 0;                  \
    PARM->path = ".";                                                   \
    PARM->cf = csound->expanded_sco;                                    \
  }
static MACRO *find_definition(MACRO *, char *);
#line 928 "Engine/csound_prslex.c"

#line 930 "Engine/csound_prslex.c"

#define INITIAL 0
#define incl 1
#define macro 2
#define umacro 3
#define ifdef 4
#define lname 5

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* %if-c-only */
#include <unistd.h>
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* %if-c-only Reentrant structure and macros (non-C++). */
/* %if-reentrant */

/* Holds the entire state of the reentrant scanner. */
struct yyguts_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE * yy_buffer_stack; /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;

    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;

    }; /* end struct yyguts_t */

/* %if-c-only */

static int yy_init_globals ( yyscan_t yyscanner );

/* %endif */

/* %if-reentrant */

int yylex_init (yyscan_t* scanner);

int yylex_init_extra ( YY_EXTRA_TYPE user_defined, yyscan_t* scanner);

/* %endif */

/* %endif End reentrant structures and macros. */

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int yylex_destroy ( yyscan_t yyscanner );

int yyget_debug ( yyscan_t yyscanner );

void yyset_debug ( int debug_flag , yyscan_t yyscanner );

YY_EXTRA_TYPE yyget_extra ( yyscan_t yyscanner );

void yyset_extra ( YY_EXTRA_TYPE user_defined , yyscan_t yyscanner );

FILE *yyget_in ( yyscan_t yyscanner );

void yyset_in  ( FILE * _in_str , yyscan_t yyscanner );

FILE *yyget_out ( yyscan_t yyscanner );

void yyset_out  ( FILE * _out_str , yyscan_t yyscanner );

			int yyget_leng ( yyscan_t yyscanner );

char *yyget_text ( yyscan_t yyscanner );

int yyget_lineno ( yyscan_t yyscanner );

void yyset_lineno ( int _line_number , yyscan_t yyscanner );

int yyget_column  ( yyscan_t yyscanner );

void yyset_column ( int _column_no , yyscan_t yyscanner );

/* %if-bison-bridge */
/* %endif */

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap ( yyscan_t yyscanner );
#else
extern int yywrap ( yyscan_t yyscanner );
#endif
#endif

/* %not-for-header */
#ifndef YY_NO_UNPUT
    
    static void yyunput ( int c, char *buf_ptr  , yyscan_t yyscanner);
    
#endif
/* %ok-for-header */

/* %endif */

#ifndef yytext_ptr
static void yy_flex_strncpy ( char *, const char *, int , yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen ( const char * , yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT
/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */
#ifdef __cplusplus
static int yyinput ( yyscan_t yyscanner );
#else
static int input ( yyscan_t yyscanner );
#endif
/* %ok-for-header */

/* %endif */
#endif

/* %if-c-only */

/* %endif */

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* %if-c-only Standard (non-C++) definition */
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, (size_t) yyleng, 1, yyout )) {} } while (0)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
/* %% [5.0] fread()/read() definition of YY_INPUT goes here unless we're doing C++ \ */\
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		int n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = (int) fread(buf, 1, (yy_size_t) max_size, yyin)) == 0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\
/* %if-c++-only C++ definition \ */\
/* %endif */

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
/* %if-c-only */
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

/* %if-tables-serialization structures and prototypes */
/* %not-for-header */
/* %ok-for-header */

/* %not-for-header */
/* %tables-yydmap generated elements */
/* %endif */
/* end tables serialization structures and prototypes */

/* %ok-for-header */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1
/* %if-c-only Standard (non-C++) definition */

extern int yylex (yyscan_t yyscanner);

#define YY_DECL int yylex (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK /*LINTED*/break;
#endif

/* %% [6.0] YY_RULE_SETUP definition goes here */
#define YY_RULE_SETUP \
	if ( yyleng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->yy_at_bol = \
				(yytext[yyleng - 1] == '\n'); \
	YY_USER_ACTION

/* %not-for-header */
/** The main scanner function which does all the work.
 */
YY_DECL
{
	yy_state_type yy_current_state;
	char *yy_cp, *yy_bp;
	int yy_act;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( !yyg->yy_init )
		{
		yyg->yy_init = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yyg->yy_start )
			yyg->yy_start = 1;	/* first start state */

		if ( ! yyin )
/* %if-c-only */
			yyin = stdin;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! yyout )
/* %if-c-only */
			yyout = stdout;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! YY_CURRENT_BUFFER ) {
			yyensure_buffer_stack (yyscanner);
			YY_CURRENT_BUFFER_LVALUE =
				yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner);
		}

		yy_load_buffer_state( yyscanner );
		}

	{
/* %% [7.0] user's declarations go here */
#line 117 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"


#line 1268 "Engine/csound_prslex.c"

	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
/* %% [8.0] yymore()-related code goes here */
		yy_cp = yyg->yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yyg->yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

/* %% [9.0] code to set up and find next match goes here */
		yy_current_state = yyg->yy_start;
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)] ;
			if ( yy_accept[yy_current_state] )
				{
				yyg->yy_last_accepting_state = yy_current_state;
				yyg->yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 132 )
					yy_c = yy_meta[yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
			++yy_cp;
			}
		while ( yy_current_state != 131 );
		yy_cp = yyg->yy_last_accepting_cpos;
		yy_current_state = yyg->yy_last_accepting_state;

yy_find_action:
/* %% [10.0] code to find the action number goes here */
		yy_act = yy_accept[yy_current_state];

		YY_DO_BEFORE_ACTION;

/* %% [11.0] code for yylineno update goes here */

do_action:	/* This label is used only to access EOF actions. */

/* %% [12.0] debug code goes here */
		if ( yy_flex_debug )
			{
			if ( yy_act == 0 )
				fprintf( stderr, "--scanner backing up\n" );
			else if ( yy_act < 39 )
				fprintf( stderr, "--accepting rule at line %ld (\"%s\")\n",
				         (long)yy_rule_linenum[yy_act], yytext );
			else if ( yy_act == 39 )
				fprintf( stderr, "--accepting default rule (\"%s\")\n",
				         yytext );
			else if ( yy_act == 40 )
				fprintf( stderr, "--(end of buffer or a NUL)\n" );
			else
				fprintf( stderr, "--EOF (start condition %d)\n", YY_START );
			}

		switch ( yy_act )
	{ /* beginning of action switch */
/* %% [13.0] actions go here */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yyg->yy_hold_char;
			yy_cp = yyg->yy_last_accepting_cpos;
			yy_current_state = yyg->yy_last_accepting_state;
			goto yy_find_action;

case 1:
/* rule 1 can match eol */
YY_RULE_SETUP
#line 119 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
#if 0
                  char bb[80];
#endif
                  csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),
                                       yyscanner);
#if 0
                  if (PARM->isString==0) {
                    sprintf(bb, "#sline %d ", csound_prsget_lineno(yyscanner));
                    corfile_puts(csound, bb, PARM->cf);
                  }
#endif
                }
	YY_BREAK
case 2:
/* rule 2 can match eol */
YY_RULE_SETUP
#line 132 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  corfile_putc(csound, '\n', PARM->cf);
                  csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),
                                       yyscanner);
                  csound_prs_line(PARM->cf, yyscanner);
                }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 138 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1) {
                    comment(yyscanner);
                    corfile_putc(csound, '\n', PARM->cf);
                    csound_prs_line(PARM->cf, yyscanner);
                  }
                  else {
                    corfile_puts(csound, yytext, PARM->cf);
                  }
                }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 148 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1) {
                    comment(yyscanner);
                    corfile_putc(csound, '\n', PARM->cf);
                    csound_prs_line(PARM->cf, yyscanner);
                  }
                  else {
                    corfile_puts(csound, yytext, PARM->cf);
                  }
                }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 158 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1) {
                    comment(yyscanner);
                    corfile_putc(csound, '\n', PARM->cf);
                    csound_prs_line(PARM->cf, yyscanner);
                  }
                  else {
                    corfile_puts(csound, yytext, PARM->cf);
                  }
                }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 168 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1)
                    do_comment(yyscanner);
                  else
                    corfile_puts(csound, yytext, PARM->cf);
                }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 174 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{ corfile_puts(csound, yytext, PARM->cf); }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 175 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  corfile_putc(csound, '"', PARM->cf);
                  //printf("string start/end: >>%s<<\n", PARM->cf->body);
                  PARM->isString = !PARM->isString;
                  if (PARM->isinclude && PARM->isString==0) {
                    do_new_include(csound, yyscanner);
                    PARM->isinclude = 0;
                  }
                }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 184 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                   MACRO     *mm = PARM->macros;
                   mm = find_definition(mm, yytext+1);
                   if (UNLIKELY(mm == NULL)) {
                     csound->Message(csound,Str("Undefined macro: '%s'"), yytext);
                     //csound->LongJmp(csound, 1);
                     corfile_puts(csound, "$error", PARM->cf);
                   }
                   else {
                     expand_macro(csound, mm, yyscanner);
                   }
                }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 196 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                   MACRO     *mm = PARM->macros;
                   //int err =0;
                   //char      *mname;
                   //int c, i, j, cnt=0;
                   //csound->DebugMsg(csound,"Macro with arguments call %s\n",
                   //                 yytext);
                   yytext[yyleng-1] = '\0';
                   mm = find_definition(mm, yytext+1);
                   if (UNLIKELY(mm == NULL)) {
                     csound->Message(csound,Str("Undefined macro: '%s'"), yytext);
                     corfile_puts(csound, "$error", PARM->cf);
                     //csound->LongJmp(csound, 1);
                   }
                   else {
                     expand_macroa(csound, mm, yyscanner);
                   }
                }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 214 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1)
                    PARM->isinclude = 1;
                  else
                    corfile_puts(csound, yytext, csound->expanded_orc);
                }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 220 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1)
                    BEGIN(incl);
                  else
                    corfile_puts(csound, yytext, PARM->cf);
                }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 226 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
/* eat the whitespace */
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 227 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{ /* got the include file name */
                  do_include(csound, yytext[0], yyscanner);
                  BEGIN(INITIAL);
                }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 231 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{ // special case for np3 type carries to avoid treatment as n
                 corfile_puts(csound, yytext, PARM->cf);
                }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 234 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                 //printf("exit found: >>>%s<<<\n", PARM->cf->body);
                  corfile_putc(csound, '\0', PARM->cf);
                  corfile_putc(csound, '\0', PARM->cf);
                  delete_macros(csound, yyscanner);
                  return 0;
                }
	YY_BREAK
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(incl):
case YY_STATE_EOF(macro):
case YY_STATE_EOF(umacro):
case YY_STATE_EOF(ifdef):
case YY_STATE_EOF(lname):
#line 241 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (on_EOF(csound, yyscanner)==0) yyterminate();
                }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 244 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1)
                    BEGIN(macro);
                  else
                    corfile_puts(csound, yytext, PARM->cf);
                }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 250 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
/* eat the whitespace */
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 251 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  yytext[yyleng-1] = '\0';
                  csound->DebugMsg(csound,"Define macro with args %s\n",
                                      yytext);
                  /* print_csound_prsdata(csound, "Before do_macro_arg",
                                          yyscanner); */
                  do_macro_arg(csound, yytext, yyscanner);
                  //print_csound_prsdata(csound,"After do_macro_arg", yyscanner);
                  BEGIN(INITIAL);
                }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 261 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  csound->DebugMsg(csound,"Define macro %s\n", yytext);
                  /* print_csound_prsdata(csound,"Before do_macro", yyscanner); */
                  do_macro(csound, yytext, yyscanner);
                  //print_csound_prsdata(csound,"After do_macro", yyscanner);
                  BEGIN(INITIAL);
                }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 268 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{ csound->Message(csound,
                                  Str("Unexpected character %c(%.2x) line %d\n"),
                                  yytext[0], yytext[0],
                                  csound_prsget_lineno(yyscanner));
                  csound->LongJmp(csound, 1);
                }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 274 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1)
                    BEGIN(umacro);
                  else
                    corfile_puts(csound, yytext, PARM->cf);
                }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 280 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
/* eat the whitespace */
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 281 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  csound->DebugMsg(csound,"Undefine macro %s\n", yytext);
                  do_umacro(csound, yytext, yyscanner);
                  BEGIN(INITIAL);
                }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 287 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1) {
                    PARM->isIfndef = (yytext[3] == 'n');  /* #ifdef or #ifndef */
                    csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),
                                         yyscanner);
                    corfile_putc(csound, '\n', PARM->cf);
                    csound_prs_line(PARM->cf, yyscanner);
                    BEGIN(ifdef);
                  }
                  else {
                    corfile_puts(csound, yytext, PARM->cf);
                  }
                }
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 300 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
/* eat the whitespace */
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 301 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  do_ifdef(csound, yytext, yyscanner);
                  BEGIN(INITIAL);
                }
	YY_BREAK
case 28:
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 305 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1) {
                    if (UNLIKELY(PARM->ifdefStack == NULL)) {
                      csound->Message(csound, Str("#else without #if\n"));
                      csound->LongJmp(csound, 1);
                    }
                    else if (UNLIKELY(PARM->ifdefStack->isElse)) {
                      csound->Message(csound, Str("#else after #else\n"));
                      csound->LongJmp(csound, 1);
                    }
                    PARM->ifdefStack->isElse = 1;
                    csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),
                                         yyscanner);
                    corfile_putc(csound, '\n', PARM->cf);
                    csound_prs_line(PARM->cf, yyscanner);
                    do_ifdef_skip_code(csound, yyscanner);
                  }
                  else {
                    corfile_puts(csound, yytext, PARM->cf);
                  }
                }
	YY_BREAK
case 29:
/* rule 29 can match eol */
YY_RULE_SETUP
#line 326 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString != 1) {
                    IFDEFSTACK *pp = PARM->ifdefStack;
                    if (UNLIKELY(pp == NULL)) {
                      csound->Message(csound, Str("Unmatched #end\n"));
                      csound->LongJmp(csound, 1);
                    }
                    PARM->ifdefStack = pp->prv;
                    csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),
                                         yyscanner);
                    corfile_putc(csound, '\n', PARM->cf);
                    csound_prs_line(PARM->cf, yyscanner);
                    mfree(csound, pp);
                  }
                  else {
                    corfile_puts(csound, yytext, PARM->cf);
                  }
                }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 344 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  if (PARM->isString) {
                    corfile_putc(csound, yytext[0], PARM->cf);
                    BEGIN(lname);
                  } else {
                    corfile_puts(csound, yytext, PARM->cf);
                  }
                }
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 352 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
/* eat the whitespace */
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 353 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  corfile_putc(csound, ' ', PARM->cf);
                  corfile_puts(csound, yytext, PARM->cf);
                  BEGIN(INITIAL);
                }
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 359 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
                  int c=' ', i;
                  PARM->repeat_index++;
                  if (UNLIKELY(PARM->repeat_index >= RPTDEPTH))
                    csound->Die(csound, Str("Loops are nested too deeply"));
                  PARM->repeat_mm_n[PARM->repeat_index] =
                    (MACRO*)csound->Malloc(csound, sizeof(MACRO));
                  if (UNLIKELY(PARM->repeat_mm_n[PARM->repeat_index] == NULL)) {
                    csound->Message(csound, Str("Memory exhausted"));
                    csound->LongJmp(csound, 1);
                  }
                  PARM->repeat_cnt_n[PARM->repeat_index] =
                    extract_int(csound, yyscanner, &c);
                  if (UNLIKELY(PARM->repeat_cnt_n[PARM->repeat_index] <= 0))
                    csound->Die(csound, Str("{: invalid repeat count"));
                  if (PARM->repeat_index > 1) {
                    char st[41];
                    int j;
                    for (j = 0; j < PARM->repeat_index; j++) {
                      st[j] = ' ';
                      st[j+1] = '\0';
                    }
                    if (UNLIKELY(csound->oparms->odebug))
                      csound->Message(csound, Str("%s Nested LOOP=%d Level:%d\n"),
                                      st, PARM->repeat_cnt_n[PARM->repeat_index],
                              PARM->repeat_index);
                  }
                  else {
                    if (UNLIKELY(csound->oparms->odebug))
                      csound->Message(csound, Str("External LOOP=%d Level:%d\n"),
                              PARM->repeat_cnt_n[PARM->repeat_index],
                              PARM->repeat_index);
                  }
                  while (isblank(c)) {
                    c = input(yyscanner);
                  }
                  for (i = 0; isNameChar(c, i) && i < (NAMELEN-1); i++) {
                    PARM->repeat_name_n[PARM->repeat_index][i] = c;
                    c = input(yyscanner);
                  }
                  PARM->repeat_name_n[PARM->repeat_index][i] = '\0';
                  unput(c);
                  /* Define macro for counter */
                  PARM->repeat_mm_n[PARM->repeat_index]->name =
                    csound->Malloc(csound,
                               strlen(PARM->repeat_name_n[PARM->repeat_index])+1);
                  if (UNLIKELY(PARM->repeat_mm_n[PARM->repeat_index]->name==NULL)) {
                    csound->Message(csound, Str("Memory exhausted"));
                    csound->LongJmp(csound, 1);
                  }
                  strcpy(PARM->repeat_mm_n[PARM->repeat_index]->name,
                         PARM->repeat_name_n[PARM->repeat_index]);
                  PARM->repeat_mm_n[PARM->repeat_index]->acnt = -1;
                  PARM->repeat_mm_n[PARM->repeat_index]->body =
                    csound->Calloc(csound, 16); // ensure nulls
                  PARM->repeat_mm_n[PARM->repeat_index]->body[0] = '0';
                  PARM->repeat_indx[PARM->repeat_index] = 0;
                  csound->DebugMsg(csound,"csound_prs(%d): repeat %s zero %p\n",
                                   __LINE__,
                                   PARM->repeat_name_n[PARM->repeat_index],
                                   PARM->repeat_mm_n[PARM->repeat_index]->body);
                  PARM->repeat_mm_n[PARM->repeat_index]->next = PARM->macros;
                  PARM->macros = PARM->repeat_mm_n[PARM->repeat_index];
                  while (input(yyscanner)!='\n') {}
                  PARM->cf_stack[PARM->repeat_index] = PARM->cf;
                  PARM->cf = corfile_create_w(csound);
        }
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 426 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
          int temp;
          CORFIL *bdy = PARM->cf;
          if (UNLIKELY((temp=PARM->repeat_cnt_n[PARM->repeat_index])==0)) {
            csound->Message(csound, Str("unmatched } in score\n"));
            csound->LongJmp(csound, 1);
          }
          corfile_puts(csound, "#loop", PARM->cf);
          corfile_putc(csound, '\0', PARM->cf);
          corfile_putc(csound, '\0', PARM->cf);
          PARM->cf = PARM->cf_stack[PARM->repeat_index];
          //printf("****Repeat body\n>>>%s<<<\n", bdy->body);
          PARM->repeat_mm_n[PARM->repeat_index]->acnt = 0; /* uninhibit */
          if (UNLIKELY(PARM->macro_stack_ptr +1 >= PARM->macro_stack_size )) {
            //trace_alt_stack(csound, PARM, __LINE__);
            PARM->alt_stack =
              (MACRON*)csound->ReAlloc(csound, PARM->alt_stack,
                                   sizeof(MACRON)*(PARM->macro_stack_size+=S_INC));
            if (UNLIKELY(PARM->alt_stack == NULL)) {
              csound->Message(csound, Str("Memory exhausted"));
              csound->LongJmp(csound, 1);
            }
          }
          csound->DebugMsg(csound,"csound_ps(%d): stacking line %d at %d\n",
                           __LINE__,
                           csound_prsget_lineno(yyscanner),PARM->macro_stack_ptr);
          PARM->alt_stack[PARM->macro_stack_ptr].n = 0;
          PARM->alt_stack[PARM->macro_stack_ptr].line =
            csound_prsget_lineno(yyscanner);
          PARM->alt_stack[PARM->macro_stack_ptr++].s = NULL;
          PARM->depth++;
          csound_prspush_buffer_state(YY_CURRENT_BUFFER, yyscanner);
          csound_prs_scan_string(bdy->body, yyscanner);
          PARM->cf_stack[PARM->repeat_index] = bdy;
       }
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 462 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
          //printf("#loop found\n");
          yypop_buffer_state(yyscanner);
          PARM->depth--;
          PARM->llocn = PARM->locn; PARM->locn = make_slocation(PARM);
          /* printf("Loop structures: count = %d, name = %s, indx = %d\n", */
          /*        PARM->repeat_cnt_n[PARM->repeat_index], */
          /*        PARM->repeat_name_n[PARM->repeat_index], */
          /*        PARM->repeat_indx[PARM->repeat_index]); */
          if (++PARM->repeat_indx[PARM->repeat_index] !=
              PARM->repeat_cnt_n[PARM->repeat_index]) {
            snprintf(PARM->repeat_mm_n[PARM->repeat_index]->body, 16, "%d",
                     PARM->repeat_indx[PARM->repeat_index]);
            //printf(">>%s<<\n", PARM->cf_stack[PARM->repeat_index]->body);
            PARM->depth++;
            csound_prspush_buffer_state(YY_CURRENT_BUFFER, yyscanner);
            csound_prs_scan_string(PARM->cf_stack[PARM->repeat_index]->body,
                                   yyscanner);
          }
          else {
            //printf("*** end loop\n");
            //printf(">>%s<<\n", PARM->cf->body);
            corfile_rm(csound, &PARM->cf_stack[PARM->repeat_index]);
            PARM->repeat_index--;
            PARM->macro_stack_ptr--;
          }
       }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 489 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
         if (PARM->isString) corfile_puts(csound, yytext, PARM->cf);
         else {
           int c, i;
           char buff[120];
           //printf("r detected %d\n",PARM->in_repeat_sect );
           if (PARM->in_repeat_sect==1) {
             char *tmp = strdup(yytext);
             // Previous r opcode not terminated so fake an s and redo
             //printf("unterminated r loop\n");
             for (i = yyleng-1; i >= 0; --i)
               unput(tmp[i]);
             unput('\n'); unput('s'); unput('\n');
             free(tmp);
           }
           else {
             PARM->repeat_sect_cnt = 0;
             PARM->in_repeat_sect = 1; /* Mark as recording */
             do {
               c = input(yyscanner);
             } while (isblank(c) || c=='\0');
             while (isdigit(c)) {
               PARM->repeat_sect_cnt =
                 10 * PARM->repeat_sect_cnt + c - '0';
               c = input(yyscanner);
             }
             if (UNLIKELY(PARM->repeat_sect_cnt <= 0
                          || !isspace(c))) {
               csound->Message(csound, Str("r: invalid repeat count %d"),
                               PARM->repeat_sect_cnt);
               csound->LongJmp(csound, 1);
             }
             if (UNLIKELY(csound->oparms->odebug))
               csound->Message(csound, Str("r LOOP=%d\n"), PARM->repeat_sect_cnt);
             while (isblank(c)) {
               c = input(yyscanner);
             }
             if (!isalpha(c)) { //no macro
               //printf("No macro\n");
               PARM->repeat_sect_mm =NULL;
             }
             else {
               for (i = 0; isNameChar(c, i) && i < (NAMELEN-1); i++) {
                 buff[i] = c;
                 c = input(yyscanner);
               }
               PARM->repeat_sect_mm =
                 (MACRO*)csound->Malloc(csound, sizeof(MACRO));
               if (UNLIKELY(PARM->repeat_sect_mm== NULL)) {
                 csound->Message(csound, Str("Memory exhausted"));
                 csound->LongJmp(csound, 1);
               }
               buff[i] = '\0';
               //printf("macro name %s\n", buff);
               /* Define macro for counter */
               PARM->repeat_sect_mm->name = cs_strdup(csound, buff);
               PARM->repeat_sect_mm->acnt = -1; /* inhibit */
               PARM->repeat_sect_mm->body = csound->Calloc(csound, 16);
               PARM->repeat_sect_mm->body[0] = '0';
               csound->DebugMsg(csound,"repeat %s zero %s\n",
                                buff, PARM->repeat_sect_mm->body);
               PARM->repeat_sect_mm->next = PARM->macros; /* add to chain */
               PARM->macros = PARM->repeat_sect_mm;
             }
             unput(c);
             PARM->repeat_sect_line = PARM->line;
             PARM->repeat_sect_index = 0;
             //while (input(yyscanner)!='\n') {}
             PARM->repeat_sect_cf = PARM->cf;
             PARM->cf = corfile_create_w(csound);
           }
         }
        }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 562 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{
          if (!PARM->isString) {
            int op = yytext[strlen(yytext)-1];
            if (PARM->in_repeat_sect==1) { // Record mode
              //printf("recording section end %d %c\n>>%s<<\n",
              //       PARM->in_repeat_sect, op, PARM->cf->body);
              corfile_putc(csound, 's', PARM->cf);
              corfile_putc(csound, '\n', PARM->cf);
              while (1) {
                int c = input(yyscanner);
                //printf("copying >>%c<<(%.2x)", c, c);
                if (c=='\n' ||c=='\0') break;
                if (!isblank(c)&&!isdigit(c)&&!strchr(".e+-",c)) { unput(c); break;}
                corfile_putc(csound, c, PARM->cf);
              }
              corfile_putc(csound, '\n', PARM->cf);
              //unput('\n'); unput(op);unput('\n');
              struct yyguts_t *yyg =(struct yyguts_t*)yyscanner;
              PARM->in_repeat_sect=2;
              //printf("****Repeat body\n>>>%s<<<\n", PARM->cf->body);
              if (PARM->repeat_sect_mm)
                PARM->repeat_sect_mm->acnt = 0; /* uninhibit */
              csound_prspush_buffer_state(YY_CURRENT_BUFFER, yyscanner);
              csound_prs_scan_string(PARM->cf->body, yyscanner);
              { CORFIL *tmp = PARM->cf;
                PARM->cf = PARM->repeat_sect_cf;
                PARM->repeat_sect_cf = tmp;
              }
              PARM->line = PARM->repeat_sect_line;
            }
            else if (PARM->in_repeat_sect==2) { // expand loop
              corfile_putc(csound, 's', PARM->cf);
              while (1) {
                int c = input(yyscanner);
                corfile_putc(csound, c, PARM->cf);
                if (c=='\n') break;
              }
              corfile_putc(csound, '\n', PARM->cf);
              yypop_buffer_state(yyscanner);
              PARM->llocn = PARM->locn; PARM->locn = make_slocation(PARM);
              //printf("repeat section %d %d\n",
              //       PARM->repeat_sect_index,PARM->repeat_sect_cnt);
              PARM->repeat_sect_index++;
              if (PARM->repeat_sect_index<PARM->repeat_sect_cnt) {
                if (PARM->repeat_sect_mm) {
                  snprintf(PARM->repeat_sect_mm->body, 16, "%d",
                           PARM->repeat_sect_index);
                  //printf("%s now %s\n",
                  //       PARM->repeat_sect_mm->name,PARM->repeat_sect_mm->body);
                }
                csound_prspush_buffer_state(YY_CURRENT_BUFFER, yyscanner);
                csound_prs_scan_string(PARM->repeat_sect_cf->body, yyscanner);
                PARM->line = PARM->repeat_sect_line;
              }
              else { // finished loop
                //corfile_puts(csound, 'r', PARM->cf);
                /* corfile_putc(csound, '\n', PARM->cf); */
                //printf("end of loop\n");
                //printf("****>>%s<<****\n", PARM->cf->body);
                PARM->in_repeat_sect=0;
                corfile_rm(csound, &PARM->repeat_sect_cf);
                //csound->Free(csound, PARM->repeat_sect_mm->body);
                //PARM->repeat_sect_mm->body = NULL;
              }
            }
            else {
              corfile_putc(csound, op, PARM->cf);
              int c = input(yyscanner);
              if (isblank(c)) {
                while (1) {
                  //printf("**copy %.2x(%c)\n", c, c);
                  corfile_putc(csound, c, PARM->cf);
                  if (c=='\n') break;
                  if (c=='\0') {
                    corfile_putc(csound, '\n', PARM->cf);
                    break;
                  }
                  c = input(yyscanner);
                }
              }
              corfile_putc(csound, '\n', PARM->cf);
            }
          }
          else corfile_puts(csound, yytext, PARM->cf);
        }
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 647 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
{ corfile_putc(csound, yytext[0], PARM->cf); }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 649 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"
ECHO;
	YY_BREAK
#line 2042 "Engine/csound_prslex.c"

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yyg->yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yyg->yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
/* %if-c-only */
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
/* %endif */
/* %if-c++-only */
/* %endif */
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state( yyscanner );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state , yyscanner);

			yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yyg->yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
/* %% [14.0] code to do back-up for compressed tables and set up yy_cp goes here */
				yy_cp = yyg->yy_last_accepting_cpos;
				yy_current_state = yyg->yy_last_accepting_state;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer( yyscanner ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yyg->yy_did_buffer_switch_on_eof = 0;

				if ( yywrap( yyscanner ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yyg->yy_c_buf_p =
					yyg->yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yyg->yy_c_buf_p =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of yylex */
/* %ok-for-header */

/* %if-c++-only */
/* %not-for-header */
/* %ok-for-header */

/* %endif */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
/* %if-c-only */
static int yy_get_next_buffer (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	char *source = yyg->yytext_ptr;
	int number_to_move, i;
	int ret_val;

	if ( yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr - 1);

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int yy_c_buf_p_offset =
				(int) (yyg->yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yyrealloc( (void *) b->yy_ch_buf,
							 (yy_size_t) (b->yy_buf_size + 2) , yyscanner );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = NULL;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yyg->yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			yyg->yy_n_chars, num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	if ( yyg->yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin  , yyscanner);
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yyg->yy_n_chars + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		int new_size = yyg->yy_n_chars + number_to_move + (yyg->yy_n_chars >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) yyrealloc(
			(void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf, (yy_size_t) new_size , yyscanner );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
		/* "- 2" to take care of EOB's */
		YY_CURRENT_BUFFER_LVALUE->yy_buf_size = (int) (new_size - 2);
	}

	yyg->yy_n_chars += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

/* %if-c-only */
/* %not-for-header */
    static yy_state_type yy_get_previous_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	yy_state_type yy_current_state;
	char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [15.0] code to get the start state into yy_current_state goes here */
	yy_current_state = yyg->yy_start;
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp )
		{
/* %% [16.0] code to find the next state goes here */
		YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yyg->yy_last_accepting_state = yy_current_state;
			yyg->yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 132 )
				yy_c = yy_meta[yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
/* %if-c-only */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	int yy_is_jam;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner; /* This var may be unused depending upon options. */
/* %% [17.0] code to find the next state, and perhaps do backing up, goes here */
	char *yy_cp = yyg->yy_c_buf_p;

	YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yyg->yy_last_accepting_state = yy_current_state;
		yyg->yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 132 )
			yy_c = yy_meta[yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
	yy_is_jam = (yy_current_state == 131);

	(void)yyg;
	return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_UNPUT
/* %if-c-only */

    static void yyunput (int c, char * yy_bp , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    yy_cp = yyg->yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yyg->yy_hold_char;

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		int number_to_move = yyg->yy_n_chars + 2;
		char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			yyg->yy_n_chars = (int) YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

/* %% [18.0] update yylineno here */

	yyg->yytext_ptr = yy_bp;
	yyg->yy_hold_char = *yy_cp;
	yyg->yy_c_buf_p = yy_cp;
}
/* %if-c-only */

/* %endif */
#endif

/* %if-c-only */
#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (yyscan_t yyscanner)
#else
    static int input  (yyscan_t yyscanner)
#endif

/* %endif */
/* %if-c++-only */
/* %endif */
{
	int c;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	*yyg->yy_c_buf_p = yyg->yy_hold_char;

	if ( *yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			/* This was really a NUL. */
			*yyg->yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr);
			++yyg->yy_c_buf_p;

			switch ( yy_get_next_buffer( yyscanner ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin , yyscanner);

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap( yyscanner ) )
						return 0;

					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput(yyscanner);
#else
					return input(yyscanner);
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yyg->yy_c_buf_p;	/* cast for 8-bit char's */
	*yyg->yy_c_buf_p = '\0';	/* preserve yytext */
	yyg->yy_hold_char = *++yyg->yy_c_buf_p;

/* %% [19.0] update BOL and yylineno */
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = (c == '\n');

	return c;
}
/* %if-c-only */
#endif	/* ifndef YY_NO_INPUT */
/* %endif */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
/* %if-c-only */
    void yyrestart  (FILE * input_file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! YY_CURRENT_BUFFER ){
        yyensure_buffer_stack (yyscanner);
		YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner);
	}

	yy_init_buffer( YY_CURRENT_BUFFER, input_file , yyscanner);
	yy_load_buffer_state( yyscanner );
}

/* %if-c++-only */
/* %endif */

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	/* TODO. We should be able to replace this entire function body
	 * with
	 *		yypop_buffer_state();
	 *		yypush_buffer_state(new_buffer);
     */
	yyensure_buffer_stack (yyscanner);
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	yy_load_buffer_state( yyscanner );

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yyg->yy_did_buffer_switch_on_eof = 1;
}

/* %if-c-only */
static void yy_load_buffer_state  (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
/* %if-c-only */
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
/* %endif */
/* %if-c++-only */
/* %endif */
	yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
/* %if-c-only */
    YY_BUFFER_STATE yy_create_buffer  (FILE * file, int  size , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state ) , yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yyalloc( (yy_size_t) (b->yy_buf_size + 2) , yyscanner );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file , yyscanner);

	return b;
}

/* %if-c++-only */
/* %endif */

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void yy_delete_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yyfree( (void *) b->yy_ch_buf , yyscanner );

	yyfree( (void *) b , yyscanner );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
/* %if-c-only */
    static void yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */

{
	int oerrno = errno;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	yy_flush_buffer( b , yyscanner);

/* %if-c-only */
	b->yy_input_file = file;
/* %endif */
/* %if-c++-only */
/* %endif */
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

/* %if-c-only */

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
/* %endif */
/* %if-c++-only */
/* %endif */
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void yy_flush_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		yy_load_buffer_state( yyscanner );
}

/* %if-c-or-c++ */
/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void yypush_buffer_state (YY_BUFFER_STATE new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (new_buffer == NULL)
		return;

	yyensure_buffer_stack(yyscanner);

	/* This block is copied from yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		yyg->yy_buffer_stack_top++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from yy_switch_to_buffer. */
	yy_load_buffer_state( yyscanner );
	yyg->yy_did_buffer_switch_on_eof = 1;
}
/* %endif */

/* %if-c-or-c++ */
/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void yypop_buffer_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (!YY_CURRENT_BUFFER)
		return;

	yy_delete_buffer(YY_CURRENT_BUFFER , yyscanner);
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if (yyg->yy_buffer_stack_top > 0)
		--yyg->yy_buffer_stack_top;

	if (YY_CURRENT_BUFFER) {
		yy_load_buffer_state( yyscanner );
		yyg->yy_did_buffer_switch_on_eof = 1;
	}
}
/* %endif */

/* %if-c-or-c++ */
/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
/* %if-c-only */
static void yyensure_buffer_stack (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	yy_size_t num_to_alloc;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if (!yyg->yy_buffer_stack) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
      num_to_alloc = 1; /* After all that talk, this was set to 1 anyways... */
		yyg->yy_buffer_stack = (struct yy_buffer_state**)yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state*));

		yyg->yy_buffer_stack_max = num_to_alloc;
		yyg->yy_buffer_stack_top = 0;
		return;
	}

	if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1){

		/* Increase the buffer to prepare for a possible push. */
		yy_size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
		yyg->yy_buffer_stack = (struct yy_buffer_state**)yyrealloc
								(yyg->yy_buffer_stack,
								num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state*));
		yyg->yy_buffer_stack_max = num_to_alloc;
	}
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_buffer  (char * base, yy_size_t  size , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return NULL;

	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state ) , yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = (int) (size - 2);	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = NULL;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b , yyscanner );

	return b;
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string (const char * yystr , yyscan_t yyscanner)
{
    
	return yy_scan_bytes( yystr, (int) strlen(yystr) , yyscanner);
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes  (const char * yybytes, int  _yybytes_len , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = (yy_size_t) (_yybytes_len + 2);
	buf = (char *) yyalloc( n , yyscanner );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n , yyscanner);
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}
/* %endif */

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

/* %if-c-only */
static void yynoreturn yy_fatal_error (const char* msg , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}
/* %endif */
/* %if-c++-only */
/* %endif */

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = yyg->yy_hold_char; \
		yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
		yyg->yy_hold_char = *yyg->yy_c_buf_p; \
		*yyg->yy_c_buf_p = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/* %if-c-only */
/* %if-reentrant */

/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE yyget_extra  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyextra;
}

/* %endif */

/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int yyget_lineno  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yylineno;
}

/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int yyget_column  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yycolumn;
}

/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *yyget_in  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyin;
}

/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *yyget_out  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyout;
}

/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int yyget_leng  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyleng;
}

/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *yyget_text  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yytext;
}

/* %if-reentrant */

/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void yyset_extra (YY_EXTRA_TYPE  user_defined , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyextra = user_defined ;
}

/* %endif */

/** Set the current line number.
 * @param _line_number line number
 * @param yyscanner The scanner object.
 */
void yyset_lineno (int  _line_number , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* lineno is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "yyset_lineno called with no buffer" );
    
    yylineno = _line_number;
}

/** Set the current column.
 * @param _column_no column number
 * @param yyscanner The scanner object.
 */
void yyset_column (int  _column_no , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* column is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "yyset_column called with no buffer" );
    
    yycolumn = _column_no;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see yy_switch_to_buffer
 */
void yyset_in (FILE *  _in_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyin = _in_str ;
}

void yyset_out (FILE *  _out_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyout = _out_str ;
}

int yyget_debug  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yy_flex_debug;
}

void yyset_debug (int  _bdebug , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yy_flex_debug = _bdebug ;
}

/* %endif */

/* %if-reentrant */
/* Accessor methods for yylval and yylloc */

/* %if-bison-bridge */
/* %endif */

/* User-visible API */

/* yylex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */
int yylex_init(yyscan_t* ptr_yy_globals)
{
    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) yyalloc ( sizeof( struct yyguts_t ), NULL );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    return yy_init_globals ( *ptr_yy_globals );
}

/* yylex_init_extra has the same functionality as yylex_init, but follows the
 * convention of taking the scanner as the last argument. Note however, that
 * this is a *pointer* to a scanner, as it will be allocated by this call (and
 * is the reason, too, why this function also must handle its own declaration).
 * The user defined value in the first argument will be available to yyalloc in
 * the yyextra field.
 */
int yylex_init_extra( YY_EXTRA_TYPE yy_user_defined, yyscan_t* ptr_yy_globals )
{
    struct yyguts_t dummy_yyguts;

    yyset_extra (yy_user_defined, &dummy_yyguts);

    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) yyalloc ( sizeof( struct yyguts_t ), &dummy_yyguts );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in
    yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    yyset_extra (yy_user_defined, *ptr_yy_globals);

    return yy_init_globals ( *ptr_yy_globals );
}

/* %endif if-c-only */

/* %if-c-only */
static int yy_init_globals (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from yylex_destroy(), so don't allocate here.
     */

    yyg->yy_buffer_stack = NULL;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = NULL;
    yyg->yy_init = 0;
    yyg->yy_start = 0;

    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = NULL;
    yyout = NULL;
#endif

    /* For future reference: Set errno on error, since we are called by
     * yylex_init()
     */
    return 0;
}
/* %endif */

/* %if-c-only SNIP! this currently causes conflicts with the c++ scanner */
/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		yy_delete_buffer( YY_CURRENT_BUFFER , yyscanner );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		yypop_buffer_state(yyscanner);
	}

	/* Destroy the stack itself. */
	yyfree(yyg->yy_buffer_stack , yyscanner);
	yyg->yy_buffer_stack = NULL;

    /* Destroy the start condition stack. */
        yyfree( yyg->yy_start_stack , yyscanner );
        yyg->yy_start_stack = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * yylex() is called, initialization will occur. */
    yy_init_globals( yyscanner);

/* %if-reentrant */
    /* Destroy the main struct (reentrant only). */
    yyfree ( yyscanner , yyscanner );
    yyscanner = NULL;
/* %endif */
    return 0;
}
/* %endif */

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, const char * s2, int n , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;

	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (const char * s , yyscan_t yyscanner)
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *yyalloc (yy_size_t  size , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	return malloc(size);
}

void *yyrealloc  (void * ptr, yy_size_t  size , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;

	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return realloc(ptr, size);
}

void yyfree (void * ptr , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	free( (char *) ptr );	/* see yyrealloc() for (char *) cast */
}

/* %if-tables-serialization definitions */
/* %define-yytables   The name for this specific scanner's tables. */
#define YYTABLES_NAME "yytables"
/* %endif */

/* %ok-for-header */

#line 649 "/build/csound/src/csound-6.18.1/Engine/csound_prs.lex"

static void comment(yyscan_t yyscanner)              /* Skip until nextline */
{
    char c;
    struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
    while ((c = input(yyscanner)) != '\n' && c != '\r') { /* skip */
      if (UNLIKELY((int)c == EOF || c=='\0')) {
        YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
          YY_BUFFER_EOF_PENDING;
        return;
      }
    }
    if (c == '\r' && (c = input(yyscanner)) != '\n') {
      if (LIKELY((int)c != EOF && c!='\0'))
        unput(c);
      else
        YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
          YY_BUFFER_EOF_PENDING;
    }
    csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),yyscanner);
}

static void do_comment(yyscan_t yyscanner)         /* Skip until * and / chars */
{
    int c;
    struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
 TOP:
    c = input(yyscanner);
    switch (c) {
    NL:
    case '\n':
      csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),yyscanner);
      goto TOP;
    case '*':
    AST:
      c = input(yyscanner);
      switch (c) {
      case '*':
        goto AST;
      case '\n':
        goto NL;
      case '/':
        return;
      case EOF:
      case '\0':
        goto ERR;
      default:
        goto TOP;
      }
    case EOF:
    ERR:
      YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
        YY_BUFFER_EOF_PENDING;
      return;
    default:
      goto TOP;
    }
}

static void do_include(CSOUND *csound, int term, yyscan_t yyscanner)
{
    char buffer[100];
    int p=0;
    int c;
    CORFIL *cf;
    struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
    while ((c=input(yyscanner))!=term) {
      if (c=='\n' || c==EOF || c=='\0') {
        csound->Warning(csound, Str("Ill formed #include ignored"));
        return;
      }
      buffer[p] = c;
      p++;
    }
    buffer[p] = '\0';
    while ((c=input(yyscanner))!='\n');
    if (UNLIKELY(PARM->depth++>=1024)) {
      csound->Die(csound, Str("Includes nested too deeply"));
    }
    csound_prsset_lineno(1+csound_prsget_lineno(yyscanner), yyscanner);
    csound->DebugMsg(csound,"line %d at end of #include line\n",
                     csound_prsget_lineno(yyscanner));
    {
      uint8_t n = file_to_int(csound, buffer);
      //char bb[128];
      PARM->lstack[PARM->depth] = n;
      //sprintf(bb, "#source %llu\n", PARM->locn = make_slocation(PARM));
      PARM->llocn = PARM->locn;
#ifdef SCORE_PARSER
      //corfile_puts(csound, bb, PARM->cf);
#endif
    }
    // printf("reading included file \"%s\"\n", buffer);
    csound->DebugMsg(csound,"reading included file \"%s\"\n", buffer);
    if (UNLIKELY(isDir(buffer)))
      csound->Warning(csound, Str("%s is a directory; not including"), buffer);
    csound->DebugMsg(csound, "path = %s\n", PARM->path);
    if (PARM->path && buffer[0]!= DIRSEP) { // if nested included directories
      char tmp[1024];
      csound->DebugMsg(csound, "using path %s\n", PARM->path);
      strncpy(tmp, PARM->path, 1023);
      strcat(tmp, "/");
      strncat(tmp, buffer, 1022-strlen(tmp));
      if ((cf = copy_to_corefile(csound, tmp, "INCDIR", 0))==NULL)
          cf = copy_to_corefile(csound, buffer, "INCDIR", 0);
    }
    else cf = copy_to_corefile(csound, buffer, "INCDIR", 0);
    if (UNLIKELY(cf == NULL))
      csound->Die(csound,
                  Str("Cannot open #include'd file %s\n"), buffer);
    printf("stack pointer = %d\n", PARM->macro_stack_ptr);
    if (UNLIKELY(PARM->macro_stack_ptr +1 >= PARM->macro_stack_size )) {
      //trace_alt_stack(csound, PARM, __LINE__);
      PARM->alt_stack =
        (MACRON*) csound->ReAlloc(csound, PARM->alt_stack,
                                  sizeof(MACRON)*(PARM->macro_stack_size+=S_INC));
      if (UNLIKELY(PARM->alt_stack == NULL)) {
        csound->Message(csound, Str("Memory exhausted"));
        csound->LongJmp(csound, 1);
      }
      csound->DebugMsg(csound, "alt_stack now %d long,\n",
                       PARM->macro_stack_size);
    }
    csound->DebugMsg(csound,"csound_prs(%d): stacking line %d at %d\n", __LINE__,
           csound_prsget_lineno(yyscanner),PARM->macro_stack_ptr);
    PARM->alt_stack[PARM->macro_stack_ptr].n = 0;
    PARM->alt_stack[PARM->macro_stack_ptr].line = csound_prsget_lineno(yyscanner);
    if (strrchr(buffer,DIRSEP)) {
      PARM->alt_stack[PARM->macro_stack_ptr].path = PARM->path;
      //printf("setting path from %s to ", PARM->path);
      PARM->path = strdup(buffer); /* wasteful! */
      *(strrchr(PARM->path,DIRSEP)) = '\0';
      //printf("%s\n",PARM->path);
    }
    else PARM->alt_stack[PARM->macro_stack_ptr].path = NULL;
    PARM->alt_stack[PARM->macro_stack_ptr++].s = NULL;
    csound_prspush_buffer_state(YY_CURRENT_BUFFER, yyscanner);
    csound_prs_scan_string(cf->body, yyscanner);
    corfile_rm(csound, &cf);
    csound->DebugMsg(csound,"Set line number to 1\n");
    csound_prsset_lineno(1, yyscanner);
}

void  do_new_include(CSOUND *csound, yyscan_t yyscanner)
{
    char buffer[128];
    CORFIL *cf = PARM->cf;
    int p = cf->p-2;
    struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;

    //printf("*** in do_new_include\n");
    cf->body[p+1] = '\0';
    while (cf->body[p]!='"') p--;
    //printf("*** name is >>%s<<\n", &cf->body[p]);
    cf->body[p] = '\0';
    strncpy(buffer, &cf->body[p+1],127); buffer[127]='\0';
    cf->p = p;
    //printf("****buffer >>%s<<\n", buffer);
    while ((input(yyscanner))!='\n');
    if (UNLIKELY(PARM->depth++>=1024)) {
      csound->Die(csound, Str("Includes nested too deeply"));
    }
    csound_prsset_lineno(1+csound_prsget_lineno(yyscanner), yyscanner);
    csound->DebugMsg(csound,"line %d at end of #include line\n",
                     csound_prsget_lineno(yyscanner));
    {
      uint8_t n = file_to_int(csound, buffer);
      //char bb[128];
      PARM->lstack[PARM->depth] = n;
      //sprintf(bb, "#source %"PRIu64"\n", PARM->locn = make_slocation(PARM));
      PARM->llocn = PARM->locn;
      //corfile_puts(csound, bb, PARM->cf);
    }
    csound->DebugMsg(csound,"reading included file \"%s\"\n", buffer);
    if (UNLIKELY(isDir(buffer)))
      csound->Warning(csound, Str("%s is a directory; not including"), buffer);
    if (PARM->path && buffer[0]!=DIRSEP) {
      char tmp[1024];
      strncpy(tmp, PARM->path, 1023);
      strcat(tmp, "/");
      strncat(tmp, buffer, 1022-strlen(tmp));
      cf = copy_to_corefile(csound, tmp, "INCDIR", 0);
    }
    else cf = copy_to_corefile(csound, buffer, "INCDIR", 0);
    if (UNLIKELY(cf == NULL))
      csound->Die(csound,
                  Str("Cannot open #include'd file %s\n"), buffer);
    if (UNLIKELY(PARM->macro_stack_ptr +1 >= PARM->macro_stack_size )) {
      //trace_alt_stack(csound, PARM, __LINE__);
      PARM->alt_stack =
        (MACRON*) csound->ReAlloc(csound, PARM->alt_stack,
                                  sizeof(MACRON)*(PARM->macro_stack_size+=S_INC));
      if (UNLIKELY(PARM->alt_stack == NULL)) {
        csound->Message(csound, Str("Memory exhausted"));
        csound->LongJmp(csound, 1);
      }
      /* csound->DebugMsg(csound, "alt_stack now %d long,\n", */
      /*                  PARM->macro_stack_size); */
    }
    csound->DebugMsg(csound,"csound_prs(%d): stacking line %d at %d\n", __LINE__,
           csound_prsget_lineno(yyscanner),PARM->macro_stack_ptr);
    PARM->alt_stack[PARM->macro_stack_ptr].n = 0;
    PARM->alt_stack[PARM->macro_stack_ptr].line = csound_prsget_lineno(yyscanner);
    if (strrchr(buffer,DIRSEP)) {
      PARM->alt_stack[PARM->macro_stack_ptr].path = PARM->path;
      PARM->path = strdup(buffer); /* wasteful! */
      *(strrchr(PARM->path,DIRSEP)) = '\0';
    }
    else PARM->alt_stack[PARM->macro_stack_ptr].path = NULL;
    PARM->alt_stack[PARM->macro_stack_ptr++].s = NULL;
    csound_prspush_buffer_state(YY_CURRENT_BUFFER, yyscanner);
    csound_prs_scan_string(cf->body, yyscanner);
    corfile_rm(csound, &cf);
    csound->DebugMsg(csound,"Set line number to 1\n");
    csound_prsset_lineno(1, yyscanner);
}

static void do_macro_arg(CSOUND *csound, char *name0, yyscan_t yyscanner)
{
    MACRO *mm = (MACRO*) csound->Malloc(csound, sizeof(MACRO));
    int   arg = 0, i, c;
    int   size = 100;
    int mlen = 40;
    char *q = name0;
    char *mname = malloc(mlen);

    if (UNLIKELY(mm == NULL||mname == NULL)) {
      csound->Message(csound, Str("Memory exhausted"));
      csound->LongJmp(csound, 1);
    }
    mm->margs = MARGS;    /* Initial size */
    mm->name = (char*)csound->Malloc(csound, strlen(name0) + 1);
    if (UNLIKELY(mm->name == NULL)) {
      csound->Message(csound, Str("Memory exhausted"));
      csound->LongJmp(csound, 1);
    }
    strcpy(mm->name, name0);
    do {
      i = 0;
      q = name0;
      mname[i++] = '`';
      while ((c = *q++)) {
        mname[i++] = c;
        if (UNLIKELY(i==mlen))
          mname = (char *)realloc(mname, mlen+=40);
        if (UNLIKELY(mname == NULL)) {
          csound->Message(csound, Str("Memory exhausted"));
          csound->LongJmp(csound, 1);
        }
      }
      mname[i++] = '`';
      if (UNLIKELY(i==mlen)) {
          mname = (char *)realloc(mname, mlen+=40);
        if (UNLIKELY(mname == NULL)) {
          csound->Message(csound, Str("Memory exhausted"));
          csound->LongJmp(csound, 1);
        }
      }
      mname[i++] = '`';
      if (UNLIKELY(i==mlen)) {
        mname = (char *)realloc(mname, mlen+=40);
        if (UNLIKELY(mname == NULL)) {
          csound->Message(csound, Str("Memory exhausted"));
          csound->LongJmp(csound, 1);
        }
      }
      while (isspace((c = input(yyscanner))));

      while (isNameChar(c, i)) {
        mname[i++] = c;
        if (UNLIKELY(i==mlen)) {
          mname = (char *)realloc(mname, mlen+=40);
          if (UNLIKELY(mname == NULL)) {
            csound->Message(csound, Str("Memory exhausted"));
            csound->LongJmp(csound, 1);
          }
        }
        c = input(yyscanner);
      }
      mname[i] = '\0';
      mm->arg[arg] = csound->Malloc(csound, i + 1);
      if (UNLIKELY(mm->arg[arg] == NULL)) {
        csound->Message(csound, Str("Memory exhausted"));
        csound->LongJmp(csound, 1);
      }
      strcpy(mm->arg[arg++], mname);
      if (UNLIKELY(arg >= mm->margs)) {
        mm = (MACRO*) csound->ReAlloc(csound, mm, sizeof(MACRO)
                               + mm->margs * sizeof(char*));
        if (UNLIKELY(mm == NULL)) {
          csound->Message(csound, Str("Memory exhausted"));
          csound->LongJmp(csound, 1);
        }
        mm->margs += MARGS;
      }
      while (isspace(c))
        c = input(yyscanner);
    } while (c == '\'' || c == '#');
    if (UNLIKELY(c != ')')) {
      csound->Message(csound, Str("macro error\n"));
    }
    free(mname);
    c = input(yyscanner);
    while (c!='#') {
      if (UNLIKELY(c==EOF || c=='\0'))
        csound->Die(csound, Str("define macro runaway\n"));
      else if (c==';') {
        while ((c=input(yyscanner))!= '\n')
          if (UNLIKELY(c==EOF || c=='\0')) {
            csound->Die(csound, Str("define macro runaway\n"));
          }
      }
      else if (c=='/') {
        if ((c=input(yyscanner))=='/') {
          while ((c=input(yyscanner))!= '\n')
            if (UNLIKELY(c==EOF || c=='\0'))
              csound->Die(csound, Str("define macro runaway\n"));
        }
        else if (c=='*') {
          while ((c=input(yyscanner))!='*') {
          again:
            if (UNLIKELY(c==EOF || c=='\0'))
              csound->Die(csound, Str("define macro runaway\n"));
          }
          if ((c=input(yyscanner))!='/') goto again;
        }
      }
      else if (UNLIKELY(!isspace(c)))
        csound->Die(csound,
               Str("define macro unexpected character %c(0x%.2x) awaiting #\n"),
                    c, c);
      c = input(yyscanner); /* skip to start of body */
    }
    mm->acnt = arg;
    i = 0;
    mm->body = (char*) csound->Malloc(csound, 100);
    if (UNLIKELY(mm->body == NULL)) {
      csound->Message(csound, Str("Memory exhausted"));
      csound->LongJmp(csound, 1);
    }

    while ((c = input(yyscanner)) != '#') { /* read body */
      if (UNLIKELY(c == EOF || c=='\0'))
        csound->Die(csound, Str("define macro with args: unexpected EOF"));
      if (c=='$') {             /* munge macro name? */
        int n = strlen(name0)+4;
        if (UNLIKELY(i+n >= size)) {
          mm->body = csound->ReAlloc(csound, mm->body, size += 100);
          if (UNLIKELY(mm->body == NULL)) {
            csound->Message(csound, Str("Memory exhausted"));
            csound->LongJmp(csound, 1);
          }
        }
        mm->body[i] = '$'; mm->body[i+1] = '`';
        strcpy(&mm->body[i+2], name0);
        mm->body[i + n - 2] = '`'; mm->body[i + n - 1] = '`';
        i+=n;
        continue;
      }
      mm->body[i++] = c=='\r'?'\n':c;
      if (UNLIKELY(i >= size)) {
        mm->body = csound->ReAlloc(csound, mm->body, size += 100);
        if (UNLIKELY(mm->body == NULL)) {
          csound->Message(csound, Str("Memory exhausted"));
          csound->LongJmp(csound, 1);
        }
      }
      if (c == '\\') {                    /* allow escaped # */
        mm->body[i++] = c = input(yyscanner);
        if (UNLIKELY(i >= size)) {
          mm->body = csound->ReAlloc(csound, mm->body, size += 100);
          if (UNLIKELY(mm->body == NULL)) {
            csound->Message(csound, Str("Memory exhausted"));
            csound->LongJmp(csound, 1);
          }
        }
      }
      if (UNLIKELY(c == '\n' || c == '\r')) {
        csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),yyscanner);
        corfile_putc(csound, '\n', PARM->cf);
        csound_prs_line(PARM->cf, yyscanner);
      }
    }
    mm->body[i] = '\0';
    mm->next = PARM->macros;
    PARM->macros = mm;
}

static void do_macro(CSOUND *csound, char *name0, yyscan_t yyscanner)
{
    MACRO *mm = (MACRO*) csound->Malloc(csound, sizeof(MACRO));
    int   i, c;
    int   size = 100;
    if (UNLIKELY(mm == NULL)) {
      csound->Message(csound, Str("Memory exhausted"));
      csound->LongJmp(csound, 1);
    }

    mm->margs = MARGS;    /* Initial size */
    csound->DebugMsg(csound,"Macro definition for %s\n", name0);
    mm->name = (char*)csound->Malloc(csound, strlen(name0) + 1);
    if (UNLIKELY(mm->name == NULL)) {
      csound->Message(csound, Str("Memory exhausted"));
      csound->LongJmp(csound, 1);
    }
    strcpy(mm->name, name0);
    mm->acnt = 0;
    i = 0;
    while ((c = input(yyscanner)) != '#') {
      if (UNLIKELY(c==EOF || c=='\0'))
        csound->Die(csound, Str("define macro runaway\n"));
      else if (c==';') {
        while ((c=input(yyscanner))!= '\n')
          if (UNLIKELY(c==EOF || c=='\0')) {
            csound->Die(csound, Str("define macro runaway\n"));
          }
      }
      else if (c=='/') {
        if ((c=input(yyscanner))=='/') {
          while ((c=input(yyscanner))!= '\n')
            if (UNLIKELY(c==EOF || c=='\0'))
              csound->Die(csound, Str("define macro runaway\n"));
        }
        else if (c=='*') {
          while ((c=input(yyscanner))!='*') {
          again:
            if (UNLIKELY(c==EOF || c=='\0'))
              csound->Die(csound, Str("define macro runaway\n"));
          }
          if ((c=input(yyscanner))!='/') goto again;
        }
      }
      else if (UNLIKELY(!isspace(c)))
        csound->Die(csound,
                    Str("define macro unexpected character %c(0x%.2x)"
                        "awaiting #\n"),
                    c, c);
    }
    mm->body = (char*) csound->Malloc(csound, 100);
    if (UNLIKELY(mm->body == NULL)) {
      csound->Message(csound, Str("Memory exhausted"));
      csound->LongJmp(csound, 1);
    }
    while ((c = input(yyscanner)) != '#') {
      if (UNLIKELY(c == EOF || c==0))
        csound->Die(csound, Str("define macro: unexpected EOF"));
      mm->body[i++] = c=='\r'?'\n':c;
      if (UNLIKELY(i >= size)) {
        mm->body = csound->ReAlloc(csound, mm->body, size += 100);
        if (UNLIKELY(mm->body == NULL)) {
          csound->Message(csound, Str("Memory exhausted"));
          csound->LongJmp(csound, 1);
        }
      }
      if (c == '\\') {                    /* allow escaped # */
        mm->body[i++] = c = input(yyscanner);
        if (UNLIKELY(i >= size)) {
          mm->body = csound->ReAlloc(csound, mm->body, size += 100);
          if (UNLIKELY(mm->body == NULL)) {
            csound->Message(csound, Str("Memory exhausted"));
            csound->LongJmp(csound, 1);
          }
        }
      }
      if (UNLIKELY(c == '\n' || c == '\r')) {
        csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),yyscanner);
        corfile_putc(csound, '\n', PARM->cf);
        csound_prs_line(PARM->cf, yyscanner);
      }
    }
    mm->body[i] = '\0';
    csound->DebugMsg(csound,"Body #%s#\n", mm->body);
    mm->next = PARM->macros;
    PARM->macros = mm;
}

static void do_umacro(CSOUND *csound, char *name0, yyscan_t yyscanner)
{
    int i,c;
    if (UNLIKELY(csound->oparms->msglevel))
      csound->Message(csound,Str("macro %s undefined\n"), name0);
    csound->DebugMsg(csound, "macro %s undefined\n", name0);
    if (strcmp(name0, PARM->macros->name)==0) {
      MACRO *mm=PARM->macros->next;
      mfree(csound, PARM->macros->name); mfree(csound, PARM->macros->body);
      for (i=0; i<PARM->macros->acnt; i++)
        mfree(csound, PARM->macros->arg[i]);
      mfree(csound, PARM->macros); PARM->macros = mm;
    }
    else {
      MACRO *mm = PARM->macros;
      MACRO *nn = mm->next;
      while (strcmp(name0, nn->name) != 0) {
        mm = nn; nn = nn->next;
        if (UNLIKELY(nn == NULL)) {
          csound->Message(csound, Str("Undefining undefined macro"));
          csound->LongJmp(csound, 1);
        }
      }
      mfree(csound, nn->name); mfree(csound, nn->body);
      for (i=0; i<nn->acnt; i++)
        mfree(csound, nn->arg[i]);
      mm->next = nn->next; mfree(csound, nn);
    }
    while ((c=input(yyscanner)) != '\n' &&
           c != EOF && c != '\r'); /* ignore rest of line */
    csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),yyscanner);
}

static void do_ifdef(CSOUND *csound, char *name0, yyscan_t yyscanner)
{
    int c;
    MACRO *mm;
    IFDEFSTACK *pp;
    pp = (IFDEFSTACK*) csound->Calloc(csound, sizeof(IFDEFSTACK));
    if (UNLIKELY(pp == NULL)) {
      csound->Message(csound, Str("Memory exhausted"));
      csound->LongJmp(csound, 1);
    }
    pp->prv = PARM->ifdefStack;
    pp->isDef = PARM->isIfndef;
    for (mm = PARM->macros; mm != NULL; mm = mm->next) {
      if (strcmp(name0, mm->name) == 0) {
        pp->isDef ^= (unsigned char) 1;
        break;
      }
    }
    PARM->ifdefStack = pp;
    pp->isSkip = pp->isDef ^ (unsigned char) 1;
    if (pp->isSkip)
      do_ifdef_skip_code(csound, yyscanner);
    else
      while ((c = input(yyscanner)) != '\n' && c != '\r' && c != EOF);
}

static void do_ifdef_skip_code(CSOUND *csound, yyscan_t yyscanner)
{
    int i, c, nested_ifdef = 0;
    char buf[8];
    IFDEFSTACK *pp;
    /* buf = (char*)malloc(8*sizeof(char)); */
    /* if (UNLIKELY(buf == NULL)) { */
    /*   csound->Message(csound, Str("Memory exhausted")); */
    /*   csound->LongJmp(csound, 1); */
    /* } */
    pp = PARM->ifdefStack;
    c = input(yyscanner);
    for (;;) {
      while (c!='\n' && c!= '\r') {
        if (UNLIKELY(c == EOF || c=='\0')) {
          csound->Message(csound, Str("Unmatched #if%sdef\n"),
                          PARM->isIfndef ? "n" : "");
          csound->LongJmp(csound, 1);
        }
        c = input(yyscanner);
    }
      csound_prsset_lineno(1+csound_prsget_lineno(yyscanner),
                           yyscanner);
      corfile_putc(csound, '\n', PARM->cf);
      csound_prs_line(PARM->cf, yyscanner);
      while (isblank(c = input(yyscanner)));  /* eat the whitespace */
      if (c == '#') {
        for (i=0; islower(c = input(yyscanner)) && i < 7; i++)
          buf[i] = c;
        buf[i] = '\0';
        if (strcmp("end", buf) == 0 || strcmp("endif", buf) == 0) {
          if (nested_ifdef-- == 0) {
            PARM->ifdefStack = pp->prv;
            mfree(csound, pp);
            break;
          }
        }
        else if (strcmp("ifdef", buf) == 0 || strcmp("ifndef", buf) == 0) {
          nested_ifdef++;
        }
        else if (strcmp("else", buf) == 0 && nested_ifdef == 0) {
          if (UNLIKELY(pp->isElse)) {
            csound->Message(csound, Str("#else after #else\n"));
            csound->LongJmp(csound, 1);
          }
          pp->isElse = 1;
          break;
        }
      }
    }
    //free(buf);
    while (c != '\n' && c != EOF && c != '\r') c = input(yyscanner);
}

static void delete_macros(CSOUND *csound, yyscan_t yyscanner)
{
    MACRO * qq = PARM->macros;
    if (qq) {
      MACRO *mm = qq;
      while (mm) {
        csound->Free(csound, mm->body);
        csound->Free(csound, mm->name);
        qq = mm->next;
        csound->Free(csound, mm);
        mm = qq;
       }
    }
}

void cs_init_smacros(CSOUND *csound, PRS_PARM *qq, NAMES *nn)
{
    while (nn) {
      char  *s = nn->mac;
      char  *p = strchr(s, '=');
      char  *mname;
      MACRO *mm;

      if (p == NULL)
        p = s + strlen(s);
      if (UNLIKELY(csound->oparms->msglevel & 7))
        csound->Message(csound, Str("Macro definition for %*s\n"), (int)(p - s), s);
      s = strchr(s, ':') + 1;                   /* skip arg bit */
      if (UNLIKELY(s == NULL || s >= p)) {
        csound->Die(csound, Str("Invalid macro name for --smacro"));
      }
      mname = (char*) csound->Malloc(csound, (p - s) + 1);
      if (UNLIKELY(mname == NULL)) {
        csound->Message(csound, Str("Memory exhausted"));
        csound->LongJmp(csound, 1);
      }
      strncpy(mname, s, p - s);
      mname[p - s] = '\0';
      /* check if macro is already defined */
      for (mm = qq->macros; mm != NULL; mm = mm->next) {
        if (strcmp(mm->name, mname) == 0)
          break;
      }
      if (mm == NULL) {
        mm = (MACRO*) csound->Calloc(csound, sizeof(MACRO));
        if (UNLIKELY(mm  == NULL)) {
          csound->Message(csound, Str("Memory exhausted"));
          csound->LongJmp(csound, 1);
        }
        mm->name = mname;
        mm->next = qq->macros;
        qq->macros = mm;
      }
      else
        mfree(csound, mname);
      mm->margs = MARGS;    /* Initial size */
      mm->acnt = 0;
      if (*p != '\0')
        p++;
      mm->body = (char*) csound->Malloc(csound, strlen(p) + 1);
      if (UNLIKELY(mm->body == NULL)) {
        csound->Message(csound, Str("Memory exhausted"));
        csound->LongJmp(csound, 1);
      }
      strcpy(mm->body, p);
      nn = nn->next;
    }
}

static void expand_macro(CSOUND* csound, MACRO* mm, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
    /* Need to read from macro definition */
    if (mm->acnt<0) { /* Macro inhibitted */
      corfile_puts(csound, yytext, PARM->cf);
    }
    else {
      if (UNLIKELY(PARM->macro_stack_ptr+1 >=
                   PARM->macro_stack_size)) {
        //trace_alt_stack(csound, PARM, __LINE__);
        //printf("***extending macro stack %p\n", PARM->alt_stack);
        PARM->alt_stack =
          (MACRON*)
          csound->ReAlloc(csound, PARM->alt_stack,
                          sizeof(MACRON)*(PARM->macro_stack_size+=S_INC));
        if (UNLIKELY(PARM->alt_stack == NULL)) {
          csound->Message(csound, Str("Memory exhausted"));
          csound->LongJmp(csound, 1);
        }
        /* csound->DebugMsg(csound, "alt_stack now %d long\n", */
        /*                  PARM->macro_stack_size); */
      }
      PARM->alt_stack[PARM->macro_stack_ptr].n = 0;
      PARM->alt_stack[PARM->macro_stack_ptr].line =
        csound_prsget_lineno(yyscanner);
      PARM->alt_stack[PARM->macro_stack_ptr].path = NULL;
      PARM->alt_stack[PARM->macro_stack_ptr++].s = NULL;
      yypush_buffer_state(YY_CURRENT_BUFFER, yyscanner);
      csound_prsset_lineno(1, yyscanner);
      if (UNLIKELY(PARM->depth>1022)) {
        csound->Message(csound,
                        Str("macros/include nested too deep: "));
        csound->LongJmp(csound, 1);
      }
      PARM->lstack[++PARM->depth] =
        (strchr(mm->body,'\n') ?file_to_int(csound, yytext) : 63);
      yy_scan_string(mm->body, yyscanner);
      /* csound->DebugMsg(csound,"%p\n", YY_CURRENT_BUFFER); */
    }
}

static void expand_macroa(CSOUND *csound, MACRO* mm, yyscan_t yyscanner)
{
    struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
    int err =0;
    //char      *mname;
    int c, i, j, cnt=0;
    //mname = buf;
    /* Need to read from macro definition */
    csound->DebugMsg(csound,"Looking for %d args\n", mm->acnt);
    for (j = 0; j < mm->acnt; j++) {
      char  term = (j == mm->acnt - 1 ? ')' : '\'');
      /* Compatability */
      char  trm1 = (j == mm->acnt - 1 ? ')' : '#');
      MACRO *nn = (MACRO*) csound->Malloc(csound, sizeof(MACRO));
      //int   size = 100;
      if (UNLIKELY(nn == NULL)) {
        csound->Message(csound, Str("Memory exhausted"));
        csound->LongJmp(csound, 1);
      }
      nn->name = csound->Malloc(csound, strlen(mm->arg[j]) + 1);
      if (UNLIKELY(nn->name == NULL)) {
        csound->Message(csound, Str("Memory exhausted"));
        csound->LongJmp(csound, 1);
      }
      csound->DebugMsg(csound,"Arg %d: %s\n", j+1, mm->arg[j]);
      strcpy(nn->name, mm->arg[j]);
      csound->Message(csound, "defining argument %s ",
                      nn->name);
      i = 0;
      nn->body = (char*) csound->Malloc(csound, 100);
      if (UNLIKELY(nn->body == NULL)) {
        csound->Message(csound, Str("Memory exhausted"));
        csound->LongJmp(csound, 1);
      }
      while (1) {
        c = input(yyscanner);
        if (cnt==0 && ( c==term || c==trm1)) break;
        if (UNLIKELY(cnt==0 && c == ')')) {
          csound->Message(csound,
                          Str("Too few arguments to macro\n"));
          corfile_puts(csound, "$error", PARM->cf);
          err = 1; break;
        }
        if (c=='(') cnt++;
        if (c==')') cnt--;
        if (c == '\\') {
          int newc = input(yyscanner);
          if (newc != ')') nn ->body[i++] = c;
          c = newc;
        }
        if (UNLIKELY(i > 98)) {
          csound->Message(csound,
                          Str("Missing argument terminator\n%.98s"),
                          nn->body);
          corfile_puts(csound, "$error", PARM->cf);
          err = 1; break;
        }
        nn->body[i++] = c;
        /* if (UNLIKELY(i >= size)) { */
        /*   nn->body = csound->ReAlloc(csound, nn->body, */
        /*   size += 100); */
        /*   if (UNLIKELY(nn->body == NULL)) { */
        /*     csound->Message(csound, Str("Memory exhausted")); */
        /*     csound->LongJmp(csound, 1); */
        /*   } */
        /* } */
      }
      nn->body[i] = '\0';
      csound->Message(csound, "as...#%s#\n", nn->body);
      nn->acnt = 0;       /* No arguments for arguments */
      nn->next = PARM->macros;
      PARM->macros = nn;
    }
    if (!err) {
      //csound->DebugMsg(csound,"New body: ...#%s#\n", mm->body);
      if (UNLIKELY(PARM->macro_stack_ptr +1 >=
                   PARM->macro_stack_size )) {
        //trace_alt_stack(csound, PARM, __LINE__);
        PARM->alt_stack =
          (MACRON*)
          csound->ReAlloc(csound, PARM->alt_stack,
                          sizeof(MACRON)*(PARM->macro_stack_size+=S_INC));
        if (UNLIKELY(PARM->alt_stack == NULL)) {
          csound->Message(csound, Str("Memory exhausted"));
          csound->LongJmp(csound, 1);
        }
        /* csound->DebugMsg(csound, */
        /*        "macro_stack extends alt_stack to %d long\n", */
        /*                  PARM->macro_stack_size); */
      }
      PARM->alt_stack[PARM->macro_stack_ptr].n =
        PARM->macros->acnt;
      PARM->alt_stack[PARM->macro_stack_ptr].line =
        csound_prsget_lineno(yyscanner);
      PARM->alt_stack[PARM->macro_stack_ptr].path = NULL;
      PARM->alt_stack[PARM->macro_stack_ptr++].s = PARM->macros;
      PARM->alt_stack[PARM->macro_stack_ptr].n = 0;
      PARM->alt_stack[PARM->macro_stack_ptr].line =
        csound_prsget_lineno(yyscanner);
      PARM->alt_stack[PARM->macro_stack_ptr].path = NULL;
      /* printf("stacked line = %llu at %d\n", */
      /*  csound_prsget_lineno(yyscanner), */
      /*                       PARM->macro_stack_ptr-1); */
      PARM->alt_stack[PARM->macro_stack_ptr].s = NULL;
      //csound->DebugMsg(csound,"Push %p macro stack\n",
      //                 PARM->macros);
      yypush_buffer_state(YY_CURRENT_BUFFER, yyscanner);
      csound_prsset_lineno(1, yyscanner);
      if (UNLIKELY(PARM->depth>1022)) {
        csound->Message(csound,
                        Str("macros/include nested too deep: "));
        //csound->LongJmp(csound, 1);
        corfile_puts(csound, "$error", PARM->cf);
        err = 1;
      }
    }
    if (!err) {
      PARM->lstack[++PARM->depth] =
        (strchr(mm->body,'\n') ?file_to_int(csound, mm->name) : 63);
      yy_scan_string(mm->body, yyscanner);
      csound_prsset_lineno(0, yyscanner);
    }
}

static void csound_prs_line(CORFIL* cf, void *yyscanner)
{
    int n = csound_prsget_lineno(yyscanner);
    //printf("line number %d\n", n);
    /* This assumes that the initial line was not written with this system  */
    if (cf->p>0 && cf->body[cf->p-1]=='\n') {
      uint64_t locn = PARM->locn;
#if 0
      uint64_t llocn = PARM->llocn;
      if (locn != llocn) {
        //char bb[80];
        //sprintf(bb, "#source %llu\n", locn);
        //corfile_puts(bb, cf);
      }
#endif
      PARM->llocn = locn;
#ifdef SCORE_PARSER
      //if (n!=PARM->line+1) {
      //char bb[80];
        //sprintf(bb, "#line   %d\n", n);
        //printf("#line %d\n", n);
        //corfile_puts(bb, cf);
      //}
#endif
    }
    PARM->line = n;
}

static MACRO *find_definition(MACRO *mmo, char *s)
{
    MACRO *mm = mmo;
    if (s[strlen(s)-1]=='.') s[strlen(s)-1]='\0';
    else if (s[strlen(s)-2]=='.' && s[strlen(s)-1]=='(') {
      s[strlen(s)-2] = '('; s[strlen(s)-1] = '\0'; }
    //printf("****Looking for %s\n", s);
    while (mm != NULL) {  /* Find the definition */
      //printf("looking at %p(%s) body #%s#\n", mm, mm->name, mm->body);
      if (!(strcmp(s, mm->name))) break;
      mm = mm->next;
    }
    if (mm == NULL) {
      mm = mmo;
      s++;                      /* skip _ */
    looking:
      while (*s++!='`') { if (*s=='\0') return NULL; }
      if (*s++!='`') { s--; goto looking; }
      //printf("now try looking for %s\n", s);
      while (mm != NULL) {  /* Find the definition */
        //printf("looking at %p(%s) body #%s#\n", mm, mm->name, mm->body);
        if (!(strcmp(s, mm->name))) break;
        mm = mm->next;
      }
    }
    //if (mm) printf("found body #%s#%c\n****\n", mm->body, mm->acnt?'X':' ');
    return mm;
}

//#if 0

static int powintint(int x, int n)
{
    int32 ans = 1;
    if (n<0) return 0;
    while (n!=0) {
      if (n&1) ans = ans * x;
      n >>= 1;
      x = x*x;
    }
    return ans;
}

static int operate(CSOUND *csound, int a, int b, char c)
{
    int ans;

    switch (c) {
    case '+': ans = a + b; break;
    case '-': ans = a - b; break;
    case '*': ans = a * b; break;
    case '/': ans = a / b; break;
    case '%': ans = a % b; break;
    case '^': ans = powintint(a, b); break;
    case '&': ans = a & b; break;
    case '|': ans = a | b; break;
    case '#': ans = a ^ b; break;
    default:
      //csoundDie(csound, Str("Internal error op=%c"), c);
      ans = 0;
    }
    //printf("operate: %d %c %d => %d\n", a, c, b, ans);
    return ans;
}

static int bodmas(CSOUND *csound, yyscan_t yyscanner, int* term)
{
      char  stack[30];
      int   vv[30];
      char  *op = stack - 1;
      int   *pv = vv - 1;
      int   c;
      struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
      int   type = 0;  /* 1 -> expecting binary operator,')', or ']'; else 0 */
      *++op = '[';
      do {
      parseNumber:
        c = input(yyscanner);
      top:
        while (isblank(c)) c = input(yyscanner);
        //printf("bodmas: c='%c'\n", c);
        switch (c) {
        case '0': case '1': case '2': case '3': case '4':
        case '5': case '6': case '7': case '8': case '9': {
          //printf("bodmas:number\n");
          int i = 0;
          while (isdigit(c)) {
            i = 10 * i + c - '0';
            c = input(yyscanner);
          }
          //printf("Number: %d terminated by %c(%.2x)\n", i, c, c);
          *++pv = i;
          type = 1;
          if (c=='\0') {
            if (on_EOF(csound,yyscanner)==0) return 0;
            c = input(yyscanner);
          }
          goto top;
        }
        case '$': {
          MACRO *mm = PARM->macros;
          char buf[256];
          int p=0;
          int first = 1;
          while ((c=input(yyscanner))) {
            if (isalpha(c) || c=='_' || c=='`' || (!first && isdigit(c)))
              buf[p++] = c;
            else break;
            first = 0;
          }
          if (c=='.') c = input(yyscanner);
          buf[p] = '\0';
          //printf("macro: %s\n", buf);
          mm = find_definition(mm, buf);
          if (mm==NULL) return 0;
          if (c=='(') {
            expand_macroa(csound, mm, yyscanner);
            goto parseNumber;
          }
          else {
            //printf("body: >>%s<<\n", mm->body);
            unput(c);
            expand_macro(csound, mm, yyscanner);
            c = input(yyscanner);
          }
          goto top;
        }
        case '+': case '-':
          //printf("bodmas:arith %d type %d\n", c, type);
          if (!type)
            goto parseNumber;
          if (*op != '[' && *op != '(') {
            int v = operate(csound, *(pv-1), *pv, *op);
            op--; pv--;
            *pv = v;
          }
          type = 0;
          *++op = c;
          goto parseNumber;
        case '*':
        case '/':
        case '%':
          if (UNLIKELY(!type)) {
            csound->Message(csound, Str("illegal placement of operator %c in [] "
                                        "expression"), c);
            return 0;
          }
          if (*op == '*' || *op == '/' || *op == '%') {
            int v = operate(csound, *(pv-1), *pv, *op);
            op--; pv--;
            *pv = v;
          }
          type = 0;
          *++op = c;
          goto parseNumber;
        case '&':
        case '|':
        case '#':
          if (UNLIKELY(!type)) {
            csound->Message(csound, Str("illegal placement of operator %c in [] "
                                        "expression"), c);
            return 0;
          }
          if (*op == '|' || *op == '&' || *op == '#') {
            int v = operate(csound, *(pv-1), *pv, *op);
            op--; pv--;
            *pv = v;
          }
          type = 0;
          *++op = c;
          goto parseNumber;
        case '(':
          //printf("bodmas: bra\n");
          if (UNLIKELY(type)) {
            csound->Message(csound,
                            Str("illegal placement of '(' in [] expression"));
            return 0;
          }
          type = 0;
          *++op = c;
          goto parseNumber;
        case ')':
          //printf("bodmas:bra switch close\n");
          if (UNLIKELY(!type)) {
            csound->Message(csound,
                            Str("missing operand before ')' in [] expression"));
            return 0;
          }
          while (*op != '(') {
            int v = operate(csound, *(pv-1), *pv, *op);
            op--; pv--;
            *pv = v;
          }
          type = 1;
          op--;
          goto parseNumber;
        //        case '^':
        //type = 0;
        // *++op = c; c = getscochar(csound, 1); break;
        case '[':
          if (UNLIKELY(type)) {
            csound->Message(csound,
                            Str("illegal placement of '[' in [] expression"));
            return 0;
          }
          type = 1;
          {
            //int i;
            //MYFLT x;
            //for (i=0;i<=pv-vv;i++) printf(" %d ", vv[i]);
            //printf("| %ld\n", pv-vv);
            *++pv = bodmas(csound, yyscanner, term);
            //printf("recursion gives %lf (%d)\n", x,*(pv-1));
            //for (i=0;i<pv-vv;i++) printf(" %d ", vv[i]); printf("| %ld\n", pv-vv);
            c = input(yyscanner);
            break;
          }
        case ']':
          //printf("] case: type = %d *op=%c\n", type, *op);
          if (UNLIKELY(!type)) {
            csound->Message(csound,
                            Str("missing operand before closing bracket in []"));
            return 0;
          }
          while (*op != '[') {
            int v = operate(csound, *(pv-1), *pv, *op);
            op--; pv--;
            *pv = v;
          }
          //printf("done ]*** *op=%c v=%d (%c)\n", *op, *pv, c);
          *term = '~'; return *pv;
        case '~':
          break;
        case ' ':               /* Ignore spaces */
          c = input(yyscanner);
          continue;
        case EOF:
        case '\0':
          if (on_EOF(csound, yyscanner)==0) {
            yyterminate(); return 0;
          }
          c = input(yyscanner);
          goto top;
        default:
          csound->Message(csound,
                          Str("illegal character %c(%.2x) in [] expression"),
                          c, c);
          return 0;
        }
      } while (c != '~');
      return *pv;
}
//#endif

static int extract_int(CSOUND *csound, yyscan_t yyscanner, int* term)
{
    int c;
 top:
    do {
      c = input(yyscanner);
    } while (isblank(c));
    if (c=='$') { // macro to yield count
      char buf[256];
      int i=0;
      MACRO* mm;
      //printf("*** macro count\n");
      while (isalnum(c=input(yyscanner)) || c=='_') {
        buf[i++] = c;
      }
      if (c=='.') c = input(yyscanner);
      buf[i] = '\0';
      //printf("*** lookup macro %s\n", buf);
      if ((mm = find_definition(PARM->macros, buf))==NULL) {
        csound->Message(csound,Str("Undefined macro: '%s'"), buf);
        //csound->LongJmp(csound, 1);
        corfile_puts(csound, "$error", PARM->cf);
        *term = c;
        return 0;
      }
      else if (c=='(') {
        struct yyguts_t *yyg =(struct yyguts_t*)yyscanner;
        unput(c);
        expand_macroa(csound, mm, yyscanner);
        goto top;
      }
      else {
        *term = c;
        return atoi(mm->body);
      }
    }
    else if (c=='[') {
      //printf("** ] case\n");
      int n =  bodmas(csound, yyscanner,term);
      return n;
    }
    else if (c=='\0') {
      if (on_EOF(csound, yyscanner)==0) return 0;
      goto top;
    }
    else {
      int i = 0;
      while (isdigit(c)) {
        i = 10 * i + c - '0';
        c = input(yyscanner);
      }
      *term = c;
      return i;
    }
    *term = c;
    return 0;
}

static int on_EOF(CSOUND* csound, void* yyscanner)
{
    MACRO *x, *y=NULL;
    int n;
    struct yyguts_t *yyg =(struct yyguts_t*)yyscanner;
    csound->DebugMsg(csound,"*********Leaving buffer %p\n",
                     YY_CURRENT_BUFFER);
    //printf("stack pointer = %d\n", PARM->macro_stack_ptr);
    yypop_buffer_state(yyscanner);
    PARM->depth--;
    if (UNLIKELY(PARM->depth > 1024)) {
      //csound->Die(csound, Str("unexpected EOF!!"));
      csound->Message(csound, Str("unexpected EOF!!\n"));
      csound->LongJmp(csound, 1);
    }
    PARM->llocn = PARM->locn; PARM->locn = make_slocation(PARM);
    csound->DebugMsg(csound,"csound-prs(%d): loc=%u; lastloc=%u\n",
                     __LINE__, PARM->llocn, PARM->locn);
    if ( !YY_CURRENT_BUFFER ) return 0;
    csound->DebugMsg(csound,"End of input; popping to %p\n",
                     YY_CURRENT_BUFFER);
    csound_prs_line(PARM->cf, yyscanner);
    n = PARM->alt_stack[--PARM->macro_stack_ptr].n;
    if (PARM->alt_stack[PARM->macro_stack_ptr].path) {
      //printf("restoring path from %s to %s\n",
      //    PARM->path, PARM->alt_stack[PARM->macro_stack_ptr].path);
      free(PARM->path);
      PARM->path = PARM->alt_stack[PARM->macro_stack_ptr].path;
    }
    /* printf("lineno on stack is %llu\n", */
    /*        PARM->alt_stack[PARM->macro_stack_ptr].line); */
    csound->DebugMsg(csound,"n=%d\n", n);
    if (n!=0) {
      //printf("We need to delete %d macros starting with %d\n",
      //       n, PARM->macro_stack_ptr);
      y = PARM->alt_stack[PARM->macro_stack_ptr].s;
      x = PARM->macros;
      if (x==y) {
        while (n>0) {
          mfree(csound, y->name); x=y->next;
          mfree(csound, y); y=x; n--;
        }
        PARM->macros = x;
      }
      else {
        MACRO *nxt = y->next;
        while (x->next != y) x = x->next;
        while (n>0) {
          nxt = y->next;
          mfree(csound, y->name); mfree(csound, y); y=nxt; n--;
        }
        x->next = nxt;
      }
      y->next = x;
    }
    csound_prsset_lineno(PARM->alt_stack[PARM->macro_stack_ptr].line,
                         yyscanner);
    /* csound->DebugMsg(csound, "csound_prs(%d): line now %d at %d\n", */
    /*                  __LINE__, */
    /*                  csound_prsget_lineno(yyscanner), */
    /*                  PARM->macro_stack_ptr); */
    csound->DebugMsg(csound,
                     "End of input segment: macro pop %p -> %p\n",
                     y, PARM->macros);
    csound_prsset_lineno(PARM->alt_stack[PARM->macro_stack_ptr].line,
                         yyscanner);
    //print_csound_prsdata(csound,"Before prs_line", yyscanner);
    csound_prs_line(PARM->cf, yyscanner);
    //print_csound_prsdata(csound,"After prs_line", yyscanner);
    return 1;
}

/* static void trace_alt_stack(CSOUND* csound, PRS_PARM* p, int line) */
/* { */
/*  printf("***Line %d extend alt_stack from %d/%d (%p)\n", */
/*         line, p->macro_stack_ptr, p->macro_stack_size, p->alt_stack); */
/* } */

#if 0
static void print_csound_prsdata(CSOUND *csound, char *mesg, void *yyscanner)
{
    struct yyguts_t *yyg =(struct yyguts_t*)yyscanner;
    csound->DebugMsg(csound,"********* %s extra data ************", mesg);
    csound->DebugMsg(csound,"yyscanner = %p", yyscanner);
    csound->DebugMsg(csound,"yyextra_r = %p, yyin_r = %p, yyout_r = %p,"
                     " yy_buffer_stack_top = %d",
           yyg->yyextra_r, yyg->yyin_r,yyg->yyout_r, yyg->yy_buffer_stack_top);
    csound->DebugMsg(csound,"yy_buffer_stack_max = %d1, yy_buffer_stack = %p, "
                     "yy_hold_char = %d '%c'",
           yyg->yy_buffer_stack_max, yyg->yy_buffer_stack, yyg->yy_hold_char,
           yyg->yy_hold_char);
    csound->DebugMsg(csound,"yy_n_chars = %d, yyleng_r = %d, yy_c_buf_p = %p %c",
           yyg->yy_n_chars, yyg->yyleng_r, yyg->yy_c_buf_p, *yyg->yy_c_buf_p);
    csound->DebugMsg(csound,"yy_init = %d, yy_start = %d, "
                     "yy_did_buffer_switch_on_eof = %d",
           yyg->yy_init, yyg->yy_start, yyg->yy_did_buffer_switch_on_eof);
    csound->DebugMsg(csound,"yy_start_stack_ptr = %d,"
                     " yy_start_stack_depth = %d, yy_start_stack = %p",
           yyg->yy_start_stack_ptr, yyg->yy_start_stack_depth, yyg->yy_start_stack);

    csound->DebugMsg(csound,"yy_last_accepting_state = %d, "
                     "yy_last_accepting_cpos = %p %c",
           yyg->yy_last_accepting_state, yyg->yy_last_accepting_cpos,
                     *yyg->yy_last_accepting_cpos);
    csound->DebugMsg(csound,"yylineno_r = %d, yy_flex_debug_r = %d, "
                     "yytext_r = %p \"%s\", yy_more_flag = %d, yy_more_len = %d",
           yyg->yylineno_r, yyg->yy_flex_debug_r, yyg->yytext_r, yyg->yytext_r,
                     yyg->yy_more_flag, yyg->yy_more_len);
    {
      PRS_PARM* pp = yyg->yyextra_r;
      printf("macros = %p, isIfndef = %d, isString = %d, line - %d loc = %d\n",
             pp->macros, pp->isIfndef, pp->isString, pp->line, pp->locn);
      printf("llocn = %d dept=%d\n", pp->llocn, pp->depth);
    }
    csound->DebugMsg(csound,"*********\n");
}
#endif

